<?php

namespace App\DataTables;

use App\Models\Car;
use Form;
use Yajra\Datatables\Services\DataTable;

class CarDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->addColumn('action', 'cars.datatables_actions')
            // ->rawColumns(["action", "owner_id"])
            // ->editColumn('owner_id', function ($data) {
            //     // dd($data->owner->name);
            //     return $data->owner->name;
            // })
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $owner = $this->request()->get('owner');
        
        $cars = Car::query()->with("owner");

        if ($owner != '') {
            $cars->where('car.owner_id', '=', $owner);
        }

        return $this->applyScopes($cars);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '20%'])
            ->ajax('')
            ->parameters([
                'responsive'=>true,
                'language' => [
                    'url' => url('vendor\datatables\lang\es.json')
                ],
                'order' => [0, 'asc'],
                "lengthMenu" => [[10, 25, 50, 100, -1], [10, 25, 50, 100, "Todos"]],
                'dom' => 'lBfrtip',
                'scrollX' => false,
                'buttons' => [
                    ["extend" => "reload", "text" => "<i class='fa fa-refresh'></i> Recargar"],
                    [
                        'extend' => 'collection',
                        'text' => '<i class="fa fa-download"></i> Exportar',
                        'buttons' => [
                            'csv',
                            'excel',
                            'pdf',
                        ],
                    ],
                    ["extend" => "print", "text" => "<i class='fa fa-print'></i> Imprimir"],
                ]
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        return [
            'Id_Auto' => ['name' => 'name', 'data' => 'name'],
            'Descripción' => ['name' => 'description', 'data' => 'description'],
            // 'owner' => ['name' => 'owner_id', 'data' => 'owner_id'],
            'Dueño' => ['name' => 'owner.name', 'data' => 'owner.name']
        ];
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'cars';
    }
}
