<?php

namespace App\DataTables;

use App\Models\CashOutbound;
use Carbon\Carbon;
use Form;
use Yajra\Datatables\Services\DataTable;

class CashOutboundDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->addColumn('action', 'cash_outbounds.datatables_actions')
            ->editColumn("user_shift_id",function ($data){
                $html = "<div class='row'>
                        <div class='col-lg-4 bg-green-400 m-0 center-block'>
                            <i class='fa fa-2x fa-clock-o text-white m-2 '></i>
                            <label>
                                <span class='text-white f-s-11'>
                                    Turno: ". $data["user_shift_id"].
                    "</span>
                            </label>  
                        </div>
                        <div class='col-lg-8 m-0'>";
                if(isset($data->userShift->final_shift_date)) {
                    $html .= "<span class='label label-success'>Inicio de Turno: " .
                        Carbon::createFromFormat('Y-m-d H:i:s', $data->userShift->start_shift_date)->format("Y-m-d")
                        . "</span><br/>
                                <span class='label label-danger'>Fin de Turno: " .
                        Carbon::createFromFormat('Y-m-d H:i:s', $data->userShift->final_shift_date)->format("Y-m-d")
                        . "</span>
                            </div>";
                }else{
                    $html .= "<span class='label label-success'>Turno en curso</span>
                        </div>";
                }

                $html.="</div>";
                return $html;
            })
            ->rawColumns(['action', 'user_shift_id'])
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $cashOutbounds = CashOutbound::with("car")
            ->with("userShift")->with("userShift.user");

        return $this->applyScopes($cashOutbounds);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '120px'])
            ->ajax('')
            ->parameters([
                'dom' => 'Bfrtip',
                'scrollX' => false,
                'buttons' => [
                    'print',
                    'reset',
                    'reload',
                    [
                         'extend'  => 'collection',
                         'text'    => '<i class="fa fa-download"></i> Export',
                         'buttons' => [
                             'csv',
                             'excel',
                             'pdf',
                         ],
                    ],
                    'colvis'
                ]
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        return [
            'Turno' => ['name' => 'user_shift_id', 'data' => 'user_shift_id'],
            'Descripción' => ['name' => 'description', 'data' => 'description'],
            'Monto' => ['name' => 'amount', 'data' => 'amount'],
            'Monto Regresado' => ['name' => 'return_amount', 'data' => 'return_amount'],
            'created_at' => ['name' => 'created_at', 'data' => 'created_at'],
            'Codigó Taxi' => ['name' => 'car.name', 'data' => 'car.name']
        ];
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'cashOutbounds';
    }
}
