<?php

namespace App\DataTables;

use App\Models\CashRegister;
use Form;
use Yajra\Datatables\Services\DataTable;

class CashRegisterDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->rawColumns(["action", "active"])
            ->addColumn('action', 'cash_registers.datatables_actions')
            ->editColumn("active",function ($data){
                $checked = "";
                $route = route("cashRgisters.changeStatus",$data["active"]);
                if (($data->active)) {
                    $checked = "checked";
                }
                return <<<HTML
                    <label class="container">
                        <input type="checkbox" name="changeActive" id="changeActive" $checked data-route="$route" data-cashRegister="{$data->id}">
                        <span class="checkmark"></span>
                    </label>
HTML;
            })
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $cashRegisters = CashRegister::query();

        return $this->applyScopes($cashRegisters);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '120px'])
            ->ajax('')
            ->parameters([
                'dom' => 'Bfrtip',
                'scrollX' => false,
                'buttons' => [
                    'print',
                    'reset',
                    'reload',
                    [
                         'extend'  => 'collection',
                         'text'    => '<i class="fa fa-download"></i> Export',
                         'buttons' => [
                             'csv',
                             'excel',
                             'pdf',
                         ],
                    ],
                    'colvis'
                ]
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        return [
            'code' => ['name' => 'code', 'data' => 'code'],
            'active' => ['name' => 'active', 'data' => 'active'],
            'created_at' => ['name' => 'created_at', 'data' => 'created_at']
        ];
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'cashRegisters';
    }
}
