<?php

namespace App\DataTables;

use App\Models\ExchangeRate;
use App\Models\Utilities;
use Form;
use Yajra\Datatables\Services\DataTable;

class ExchangeRateDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            // ->addColumn('action', 'exchange_rates.datatables_actions')
            ->rawColumns(["status", "date", "coin_id"])
            ->editColumn('coin_id', function ($data) {
                return "<span class='badge badge-inverse'>" . $data->coin->key . "</span> " . $data->coin->description;
            })
            ->editColumn('date', function ($data) {
                return Utilities::formatDate($data->date);
            })
            ->editColumn('status', function ($data) {
                $class = $data->status == '1' ? 'badge-success' : 'badge-danger';
                $text = $data->status == '1' ? 'Activo' : 'Inactivo';

                return "<span class='badge " . $class . "'>" . $text . "</span>";
            })
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $coin = $this->request()->get('coin');
        $status = $this->request()->get('status');
        
        $exchangeRates = ExchangeRate::query()->with('coin');

        if (!empty($coin[0]))
            $exchangeRates->whereIn('exchange_rate.coin_id', $coin);

        if ($status != '') {
            $exchangeRates->where('exchange_rate.status', '=', $status);
        }

        return $this->applyScopes($exchangeRates);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            // ->addAction(['width' => '120px'])
            ->ajax('')
            ->parameters([
                'responsive'=>true,
                'language' => [
                    'url' => url('vendor\datatables\lang\es.json')
                ],
                'order' => [0, 'asc'],
                "lengthMenu" => [[10, 25, 50, 100, -1], [10, 25, 50, 100, "Todos"]],
                'dom' => 'lBfrtip',
                'scrollX' => false,
                'buttons' => [
                    ["extend" => "reload", "text" => "<i class='fa fa-refresh'></i> Recargar"],
                ]
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        return [
            'Moneda' => ['name' => 'coin_id', 'data' => 'coin_id'],
            'Tipo_de_Cambio' => ['name' => 'exchange_rate', 'data' => 'exchange_rate', 'class' => 'text-center'],
            'Fecha_de_Registro' => ['name' => 'date', 'data' => 'date', 'class' => 'text-center'],
            'Status' => ['name' => 'status', 'data' => 'status', 'class' => 'text-center']
        ];
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'exchangeRates';
    }
}
