<?php

namespace App\DataTables;

use App\Models\Filial;
use Form;
use Yajra\Datatables\Services\DataTable;
use Auth;

class FilialDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->rawColumns(["action", "status"])
            ->editColumn('status', function ($data) {

                if (Auth::user()->role->permissions->create_customer) {
                    switch ($data->status) {
                        case 0:
                            return '<label class="badge badge-danger">Desactivada</label>';
                        case 1:
                            return '<label class="badge badge-success">Activada</label>';
                    }
                }

                return '';
            })
            ->addColumn('action', 'filials.datatables_actions')
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $filials = Filial::query();

        return $this->applyScopes($filials);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '20%'])
            ->ajax('')
            ->parameters([
                'language' => [
                    'url' => url('vendor\datatables\lang\es.json')
                ],
                "lengthMenu" => [[10, 25, 50, 100, -1], [10, 25, 50, 100, "Todos"]],
                'dom' => 'lBfrtip',//BlfTrtip
                'scrollX' => false,
                'buttons' => [
                    ["extend" => "reload", "text" => "<i class='fa fa-refresh'></i> Recargar"],
                ]
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        return [
            'nombre' => ['name' => 'name', 'data' => 'name'],
            'serie' => ['name' => 'prefix', 'data' => 'prefix'],
            'nº_folio' => ['name' => 'number_ticket', 'data' => 'number_ticket'],
            'status' => ['name' => 'status', 'data' => 'status']
        ];
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'filials';
    }
}
