<?php

namespace App\DataTables;

use App\Models\Ticket;
use Form;
use Yajra\Datatables\Services\DataTable;
use DB;

class InvoiceHistoryDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->rawColumns(['action', 'status'])
            ->addColumn('action', 'tickets.historyTicket.datatables_actions')
            ->editColumn('date', function ($data) {
                return $data->date->format('Y-m-d');
            })
            ->editColumn('status', function ($data) {
                
                if ($data->status == 'invoiced')
                    return '<span class="badge badge-success">Vigente</span>';

                return '<span class="badge badge-danger">Cancelada</span>';
            })
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $rfc = $this->request()->get('rfc');


        $invoices = Ticket::query()
        ->select('ticket.status','ticket.shift_user_id','ticket.number_ticket','ticket.invoice','ticket.date','ticket.amount',
            'user_id','filial.related_filial','filial_id')
            ->join('customer', 'customer.id', '=', 'ticket.customer_id')
            ->join('user_shift', 'user_shift.id', '=', 'ticket.shift_user_id')
            ->join('user', 'user.id', '=', 'user_shift.user_id')
            ->join('filial', 'filial.id', '=', 'user.filial_id')
            ->whereIn('ticket.status', ["invoiced", "invoice_cancelled"]);

        if (!empty($rfc) && $rfc != strtoupper('XAXX010101000')) {
            $invoices->where('customer.rfc', '=', $rfc);
        } else
            $invoices->where('customer.rfc', '=', "NULL");

        return $this->applyScopes($invoices);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '20%'])
            ->ajax('')
            ->parameters([
                'language' => [
                    'url' => url('vendor\datatables\lang\es.json')
                ],
                'order' => [2, 'asc'],
                "lengthMenu" => [[10, 25, 50, 100, -1], [10, 25, 50, 100, "Todos"]],
                'dom' => 'lBfrtip',
                'scrollX' => true,
                'buttons' => [
                    ["extend" => "reload", "text" => "<i class='fa fa-refresh'></i> Recargar"],
                    ["extend" => "print", "text" => "<i class='fa fa-print'></i> Imprimir"],
                    [
                        'extend' => 'collection',
                        'text' => '<i class="fa fa-download"></i> Exportar',
                        'buttons' => [
                            'csv',
                            'excel',
                            'pdf',
                        ],
                    ],
                    ["extend" => "colvis", "text" => "<i class='fa fa-list-alt'></i> Seleccionar columnas"],
                ]
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        return [
            'folio' => ['name' => 'number_ticket', 'data' => 'number_ticket'],
            'factura' => ['name' => 'invoice', 'data' => 'invoice'],
            'folio_1' => ['name' => 'filial_id', 'data' => 'filial_id',"class"=>"hidden"],
            'folio_2' => ['name' => 'related_filial', 'data' => 'related_filial',"class"=>"hidden"],
            'fecha' => ['name' => 'date', 'data' => 'date'],
            'importe' => ['name' => 'amount', 'data' => 'amount'],
            'status' => ['name' => 'status', 'data' => 'status']
        ];
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'facturasticket';
    }
}
