<?php

namespace App\DataTables;

use App\Models\MethodPayment;
use Form;
use Yajra\Datatables\Services\DataTable;

class MethodPaymentDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->addColumn('action', 'method_payments.datatables_actions')
            ->rawColumns(["action", "status", "type"])
            ->editColumn('status', function ($data) {
                switch ($data->status) {
                    case 0:
                        return '<label class="badge badge-danger">Desactivada</label>';
                    case 1:
                        return '<label class="badge badge-success">Activada</label>';
                }
            })
            ->editColumn('type', function ($data) {
                switch ($data->type) {
                    case "tarjeta":
                        return '<label class="badge badge-light">Tarjeta</label>';
                    case "efectivo":
                        return '<label class="badge badge-light">Efectivo</label>';
                    case "credito":
                        return '<label class="badge badge-light">Credito</label>';
                    case "transferencia":
                        return '<label class="badge badge-light">Transferencia</label>';
                    case "aerolinea":
                        return '<label class="badge badge-light">Aerolinea</label>';
                }
            })
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $methodPayments = MethodPayment::query();

        return $this->applyScopes($methodPayments);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '25%'])
            ->ajax('')
            ->parameters([
                'responsive'=>true,
                'language' => [
                    'url' => url('vendor\datatables\lang\es.json')
                ],
                'order' => [0, 'asc'],
                "lengthMenu" => [[10, 25, 50, 100, -1], [10, 25, 50, 100, "Todos"]],
                'dom' => 'lBfrtip',
                'scrollX' => false,
                'buttons' => [
                    ["extend" => "reload", "text" => "<i class='fa fa-refresh'></i> Recargar"],
                ]
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        return [
            'Clave' => ['name' => 'key', 'data' => 'key'],
            'Descripción' => ['name' => 'description', 'data' => 'description'],
            // 'Moneda' => ['name' => 'coin_id', 'data' => 'coin_id'],
            'Tipo' => ['name' => 'type', 'data' => 'type'],
            'Status' => ['name' => 'status', 'data' => 'status']
        ];
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'methodPayments';
    }
}
