<?php

namespace App\DataTables;

use App\Models\Owner;
use Form;
use Yajra\Datatables\Services\DataTable;

class OwnerDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->addColumn('action', 'owners.datatables_actions')
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $owners = Owner::query();

        return $this->applyScopes($owners);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '20%'])
            ->ajax('')
            ->parameters([
                'responsive'=>true,
                'language' => [
                    'url' => url('vendor\datatables\lang\es.json')
                ],
                'order' => [0, 'asc'],
                "lengthMenu" => [[10, 25, 50, 100, -1], [10, 25, 50, 100, "Todos"]],
                'dom' => 'lBfrtip',
                'scrollX' => false,
                'buttons' => [
                    ["extend" => "reload", "text" => "<i class='fa fa-refresh'></i> Recargar"],
                    [
                        'extend' => 'collection',
                        'text' => '<i class="fa fa-download"></i> Exportar',
                        'buttons' => [
                            'csv',
                            'excel',
                            'pdf',
                        ],
                    ],
                    ["extend" => "print", "text" => "<i class='fa fa-print'></i> Imprimir"],
                ]
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        return [
            'Nombre' => ['name' => 'name', 'data' => 'name']
        ];
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'owners';
    }
}
