<?php

namespace App\DataTables;

use App\Models\Rate;
use Form;
use Yajra\Datatables\Services\DataTable;

class RateDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->addColumn('action', 'rates.datatables_actions')
            ->rawColumns(["action", "status"])
            ->editColumn("status", function ($data) {
                switch ($data->status) {
                    case 'foranea':
                        return '<label class="badge badge-warning">FORANEA</label>';
                    case 'local':
                        return '<label class="badge badge-primary">LOCAL</label>';
                }
            })
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $status = $this->request()->get('status');
        
        $rates = Rate::query();

        if ($status != '') {
            $rates->where('status', '=', $status);
        }

        return $this->applyScopes($rates);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '20%'])
            ->ajax('')
            ->parameters([
                'responsive'=>true,
                'language' => [
                    'url' => url('vendor\datatables\lang\es.json')
                ],
                'order' => [2, 'asc'],
                "lengthMenu" => [[10, 25, 50, 100, -1], [10, 25, 50, 100, "Todos"]],
                'dom' => 'lBfrtip',
                'scrollX' => false,
                'buttons' => [
                    ["extend" => "reload", "text" => "<i class='fa fa-refresh'></i> Recargar"],
                    [
                        'extend' => 'collection',
                        'text' => '<i class="fa fa-download"></i> Exportar',
                        'buttons' => [
                            'csv',
                            'excel',
                            'pdf',
                        ],
                    ],
                    ["extend" => "print", "text" => "<i class='fa fa-print'></i> Imprimir"],
                    ["extend" => "colvis", "text" => "<i class='fa fa-list-alt'></i> Seleccionar columnas"],
                ]
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        return [
            'Destino' => ['name' => 'destination', 'data' => 'destination'],
            'Localidad' => ['name' => 'locality', 'data' => 'locality'],
            'Costo' => ['name' => 'cost', 'data' => 'cost'],
            'Status' => ['name' => 'status', 'data' => 'status', 'class' => 'text-center']
        ];
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'rates';
    }
}
