<?php

namespace App\DataTables;

use App\Models\User;
use Form;
use Yajra\Datatables\Services\DataTable;
use Auth;

class UserDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->rawColumns(["action", "status", "role_id", 'filial_id'])
            ->addColumn('action', 'users.datatables_actions')
            ->editColumn('role_id', function ($data) {
                return $data->role->name;
            })
            ->editColumn('filial_id', function ($data) {
                if(empty($data->filial))
                    return 'NA';
                return $data->filial->name;
            })
            ->editColumn('status', function ($data) {
                switch ($data->status) {
                    case 0:
                        return '<label class="badge badge-danger">Desactivado</label>';
                    case 1:
                        return '<label class="badge badge-success">Activado</label>';
                }
            })
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $filial = $this->request()->get('filial');
        $role = $this->request()->get('role');

        $users = User::query();

        if (!empty($filial)) {
            $users->where('filial_id', '=', $filial);
        }
        if (Auth::user()->role->name != 'Super Administrador') {
            $users->where('filial_id', '=', Auth::user()->filial_id);
        }


        if (!empty($role)) {
            $users->where('role_id', '=', $role);
        }

        return $this->applyScopes($users);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '35%'])
            ->ajax('')
            ->parameters([
                'language' => [
                    'url' => url('vendor\datatables\lang\es.json')
                ],
                'order' => [1, 'asc'],
                "lengthMenu" => [[10, 25, 50, 100, -1], [10, 25, 50, 100, "Todos"]],
                'dom' => 'lBfrtip',//BlfTrtip
                'scrollX' => false,
                'buttons' => [
                    ["extend" => "reload", "text" => "<i class='fa fa-refresh'></i> Recargar"],
                ]

            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        return [
            'nombre' => ['name' => 'name', 'data' => 'name'],
            'nombre_usuario' => ['name' => 'username', 'data' => 'username'],
            'sucursal' => ['name' => 'filial_id', 'data' => 'filial_id'],
            'rol' => ['name' => 'role_id', 'data' => 'role_id'],
            'status' => ['name' => 'status', 'data' => 'status']
        ];
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'users';
    }
}
