<?php

namespace App\DataTables;

use App\Models\UserShift;
use Form;
use Illuminate\Support\Facades\Auth;
use Yajra\Datatables\Services\DataTable;

class UserShiftDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->addColumn('action', 'user_shifts.datatables_actions')
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $userShifts = UserShift::with("tickets")->with("tickets.ticketItems")->with("tickets.ticketItems.car")->with("tickets.ticketItems.car.owner")
        ->with("cashOutbound")->with("cashOutbound.car")->with("cashOutbound.car.owner")
        ->with("pay")->with("pay.ticket")->with("pay.ticket.ticketItems")->with("pay.ticket.ticketItems.car")->with("pay.ticket.ticketItems.car.owner")
        ->with("user")->with("cashRegister")
        ->where("user_id","=",Auth::user()->id);

        return $this->applyScopes($userShifts);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '120px'])
            ->ajax('')
            ->parameters([
                'dom' => 'Bfrtip',
                'scrollX' => false,
                'buttons' => [
                    'print',
                    'reset',
                    'reload',
                    [
                         'extend'  => 'collection',
                         'text'    => '<i class="fa fa-download"></i> Export',
                         'buttons' => [
                             'csv',
                             'excel',
                             'pdf',
                         ],
                    ],
                    'colvis'
                ]
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        return [
            'id' => ['name' => 'id', 'data' => 'id',"class"=>"hidden"],
            'usuario' => ['name' => 'user.name', 'data' => 'user.name'],
            'Caja' => ['name' => 'cash_register.code', 'data' => 'cash_register.code'],
            'Hora_de_inicio' => ['name' => 'start_shift_date', 'data' => 'start_shift_date'],
            'Hora_de_fin' => ['name' => 'final_shift_date', 'data' => 'final_shift_date'],
            'Monto_Inicio' => ['name' => 'init_amount', 'data' => 'init_amount'],

        ];
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'userShifts';
    }
}
