<?php

namespace App\Http\Controllers;

use App\Models\Car;
use App\Models\Owner;
use Illuminate\Database\QueryException;
use App\DataTables\CarDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateCarRequest;
use App\Http\Requests\UpdateCarRequest;
use App\Repositories\CarRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Response;

class CarController extends AppBaseController
{
    /** @var  CarRepository */
    private $carRepository;

    public function __construct(CarRepository $carRepo)
    {
        $this->carRepository = $carRepo;
    }

    /**
     * Display a listing of the Car.
     *
     * @param CarDataTable $carDataTable
     * @return Response
     */
    public function index(CarDataTable $carDataTable)
    {
        return $carDataTable->render('cars.index', [
            "owners" => Owner::get()->pluck('name', 'id')
        ]);
    }

    /**
     * Show the form for creating a new Car.
     *
     * @return Response
     */
    public function create()
    {

    }

    /**
     * Store a newly created Car in storage.
     *
     * @param CreateCarRequest $request
     *
     * @return Response
     */
    public function store(CreateCarRequest $request)
    {
        if (!Auth::user()->role->permissions->create_car)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        try {
            Car::create($request->all());

            return response()->json(["msg" => "Auto guardado correctamente."], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrió un error al guardar el auto." . $e->getMessage()], 400);
        }
    }

    /**
     * Display the specified Car.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        
    }

    /**
     * Show the form for editing the specified Car.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        
    }

    /**
     * Update the specified Car in storage.
     *
     * @param  int              $id
     * @param UpdateCarRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateCarRequest $request)
    {
        if (!Auth::user()->role->permissions->edit_car)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        $car = $this->carRepository->findWithoutFail($id);

        if (empty($car))
            return response()->json(["msg" => "No se encontró el auto."], 404);

        try {
            $car->update($request->all());

            return response()->json(["msg" => "Auto actualizado correctamente."], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrió un error al actualizar el auto." . $e->getMessage()], 400);
        }
    }

    /**
     * Remove the specified Car from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $car = $this->carRepository->findWithoutFail($id);

        if (empty($car))
            return response()->json(["msg" => "No se encontró el auto."], 404);

        try {
            $this->carRepository->delete($id);

            return response()->json(["msg" => "Auto eliminado correctamente."], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrió un error al eliminar el auto." . $e->getMessage()], 400);
        }
    }
}
