<?php

namespace App\Http\Controllers;

use App\DataTables\CashOutboundDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateCashOutboundRequest;
use App\Http\Requests\UpdateCashOutboundRequest;
use App\Models\CashOutbound;
use App\Models\UserShift;
use App\Repositories\CashOutboundRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Illuminate\Support\Facades\Auth;
use Response;

class CashOutboundController extends AppBaseController
{
    /** @var  CashOutboundRepository */
    private $cashOutboundRepository;

    public function __construct(CashOutboundRepository $cashOutboundRepo)
    {
        $this->cashOutboundRepository = $cashOutboundRepo;
    }

    /**
     * Display a listing of the CashOutbound.
     *
     * @param CashOutboundDataTable $cashOutboundDataTable
     * @return Response
     */
    public function index(CashOutboundDataTable $cashOutboundDataTable)
    {
        return $cashOutboundDataTable->render('cash_outbounds.index');
    }

    /**
     * Show the form for creating a new CashOutbound.
     *
     * @return Response
     */
    public function create()
    {
        return view('cash_outbounds.create');
    }

    /**
     * Store a newly created CashOutbound in storage.
     *
     * @param CreateCashOutboundRequest $request
     *
     * @return Response
     */
    public function store(CreateCashOutboundRequest $request)
    {
        $input = $request->all();

        $user_shift = UserShift::where("user_id","=",Auth::user()->id)
            ->where("final_shift_date","=",null)
            ->first();

        $input["user_shift_id"] = $user_shift["id"];

        $cashOutbound = $this->cashOutboundRepository->create($input);

       return response()->json(array("success"=>1,"msg"=>"Salida de dinero registrada correctamente"));
    }

    /**
     * Display the specified CashOutbound.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $cashOutbound = $this->cashOutboundRepository->findWithoutFail($id);

        if (empty($cashOutbound)) {
            Flash::error('Cash Outbound not found');

            return redirect(route('cashOutbounds.index'));
        }

        return view('cash_outbounds.show')->with('cashOutbound', $cashOutbound);
    }

    /**
     * Show the form for editing the specified CashOutbound.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $cashOutbound = $this->cashOutboundRepository->findWithoutFail($id);

        $cashOutbound = CashOutbound::where("id",$id)->with("car")->with("car.owner")->first();

        if (empty($cashOutbound)) {
            return redirect(route('cashOutbounds.index'));
        }

        return response()->json(array("success"=>"1","cashOutbound"=>$cashOutbound));
    }

    /**
     * Update the specified CashOutbound in storage.
     *
     * @param  int              $id
     * @param UpdateCashOutboundRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateCashOutboundRequest $request)
    {
        $cashOutbound = $this->cashOutboundRepository->findWithoutFail($id);

        if (empty($cashOutbound)) {
            Flash::error('Cash Outbound not found');

            return redirect(route('cashOutbounds.index'));
        }

        $cashOutbound = $this->cashOutboundRepository->update($request->all(), $id);

        Flash::success('Cash Outbound updated successfully.');

        return redirect(route('cashOutbounds.index'));
    }

    /**
     * Remove the specified CashOutbound from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $cashOutbound = $this->cashOutboundRepository->findWithoutFail($id);

        if (empty($cashOutbound)) {
            Flash::error('Cash Outbound not found');

            return redirect(route('cashOutbounds.index'));
        }

        $this->cashOutboundRepository->delete($id);

        Flash::success('Cash Outbound deleted successfully.');

        return redirect(route('cashOutbounds.index'));
    }
}
