<?php

namespace App\Http\Controllers;

use App\DataTables\CashRegisterDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateCashRegisterRequest;
use App\Http\Requests\UpdateCashRegisterRequest;
use App\Repositories\CashRegisterRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Response;

class CashRegisterController extends AppBaseController
{
    /** @var  CashRegisterRepository */
    private $cashRegisterRepository;

    public function __construct(CashRegisterRepository $cashRegisterRepo)
    {
        $this->cashRegisterRepository = $cashRegisterRepo;
    }

    /**
     * Display a listing of the CashRegister.
     *
     * @param CashRegisterDataTable $cashRegisterDataTable
     * @return Response
     */
    public function index(CashRegisterDataTable $cashRegisterDataTable)
    {
        return $cashRegisterDataTable->render('cash_registers.index');
    }

    /**
     * Show the form for creating a new CashRegister.
     *
     * @return Response
     */
    public function create()
    {
        return view('cash_registers.create');
    }

    /**
     * Store a newly created CashRegister in storage.
     *
     * @param CreateCashRegisterRequest $request
     *
     * @return Response
     */
    public function store(CreateCashRegisterRequest $request)
    {
        if (!Auth::user()->role->permissions->create_cashRegister)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        $input = $request->all();

        $cashRegister = $this->cashRegisterRepository->create($input);

        return response()->json(array("msg"=>"Caja ". $cashRegister["code"] ." regitrada correctamente",
                                        "data"=> $cashRegister));
    }

    /**
     * Display the specified CashRegister.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $cashRegister = $this->cashRegisterRepository->findWithoutFail($id);

        if (empty($cashRegister)) {
            Flash::error('Cash Register not found');

            return redirect(route('cashRegisters.index'));
        }

        return response()->json(array("data"=> $cashRegister));
    }

    /**
     * Show the form for editing the specified CashRegister.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $cashRegister = $this->cashRegisterRepository->findWithoutFail($id);

        if (empty($cashRegister)) {
            Flash::error('Cash Register not found');

            return redirect(route('cashRegisters.index'));
        }

        return view('cash_registers.edit')->with('cashRegister', $cashRegister);
    }

    /**
     * Update the specified CashRegister in storage.
     *
     * @param  int              $id
     * @param UpdateCashRegisterRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateCashRegisterRequest $request)
    {
        if (!Auth::user()->role->permissions->edit_cashRegister)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        $cashRegister = $this->cashRegisterRepository->findWithoutFail($id);

        if (empty($cashRegister)) {
            return response()->json(array("msg"=>"Caja no encontrada"),500);
        }

        $cashRegister = $this->cashRegisterRepository->update($request->all(), $id);

        return response()->json(array("msg"=>"Caja ". $cashRegister["code"] ." actualizada correctamente"),200);
    }

    /**
     * Remove the specified CashRegister from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        /*$cashRegister = $this->cashRegisterRepository->findWithoutFail($id);

        if (empty($cashRegister)) {
            Flash::error('Cash Register not found');

            return redirect(route('cashRegisters.index'));
        }

        $this->cashRegisterRepository->delete($id);

        Flash::success('Cash Register deleted successfully.');

        return redirect(route('cashRegisters.index'));*/
    }
}
