<?php

namespace App\Http\Controllers;

use App\DataTables\CashbackDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateCashbackRequest;
use App\Http\Requests\UpdateCashbackRequest;
use App\Repositories\CashbackRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Response;

class CashbackController extends AppBaseController
{
    /** @var  CashbackRepository */
    private $cashbackRepository;

    public function __construct(CashbackRepository $cashbackRepo)
    {
        $this->cashbackRepository = $cashbackRepo;
    }

    /**
     * Display a listing of the Cashback.
     *
     * @param CashbackDataTable $cashbackDataTable
     * @return Response
     */
    public function index(CashbackDataTable $cashbackDataTable)
    {
        return $cashbackDataTable->render('cashbacks.index');
    }

    /**
     * Show the form for creating a new Cashback.
     *
     * @return Response
     */
    public function create()
    {
        return view('cashbacks.create');
    }

    /**
     * Store a newly created Cashback in storage.
     *
     * @param CreateCashbackRequest $request
     *
     * @return Response
     */
    public function store(CreateCashbackRequest $request)
    {
        $input = $request->all();

        $cashback = $this->cashbackRepository->create($input);

        Flash::success('Cashback saved successfully.');

        return redirect(route('cashbacks.index'));
    }

    /**
     * Display the specified Cashback.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $cashback = $this->cashbackRepository->findWithoutFail($id);

        if (empty($cashback)) {
            Flash::error('Cashback not found');

            return redirect(route('cashbacks.index'));
        }

        return view('cashbacks.show')->with('cashback', $cashback);
    }

    /**
     * Show the form for editing the specified Cashback.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $cashback = $this->cashbackRepository->findWithoutFail($id);

        if (empty($cashback)) {
            Flash::error('Cashback not found');

            return redirect(route('cashbacks.index'));
        }

        return view('cashbacks.edit')->with('cashback', $cashback);
    }

    /**
     * Update the specified Cashback in storage.
     *
     * @param  int              $id
     * @param UpdateCashbackRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateCashbackRequest $request)
    {
        $cashback = $this->cashbackRepository->findWithoutFail($id);

        if (empty($cashback)) {
            Flash::error('Cashback not found');

            return redirect(route('cashbacks.index'));
        }

        $cashback = $this->cashbackRepository->update($request->all(), $id);

        Flash::success('Cashback updated successfully.');

        return redirect(route('cashbacks.index'));
    }

    /**
     * Remove the specified Cashback from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $cashback = $this->cashbackRepository->findWithoutFail($id);

        if (empty($cashback)) {
            Flash::error('Cashback not found');

            return redirect(route('cashbacks.index'));
        }

        $this->cashbackRepository->delete($id);

        Flash::success('Cashback deleted successfully.');

        return redirect(route('cashbacks.index'));
    }
}
