<?php

namespace App\Http\Controllers;

use DB;
use App\Models\Coin;
use App\Models\ExchangeRate;
use App\DataTables\CoinDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateCoinRequest;
use App\Http\Requests\UpdateCoinRequest;
use App\Repositories\CoinRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Response;
use Illuminate\Database\QueryException;

class CoinController extends AppBaseController
{
    /** @var  CoinRepository */
    private $coinRepository;

    public function __construct(CoinRepository $coinRepo)
    {
        $this->coinRepository = $coinRepo;
    }

    /**
     * Display a listing of the Coin.
     *
     * @param CoinDataTable $coinDataTable
     * @return Response
     */
    public function index(CoinDataTable $coinDataTable)
    {
        return $coinDataTable->render('coins.index');
    }

    /**
     * Show the form for creating a new Coin.
     *
     * @return Response
     */
    public function create()
    {
        
    }

    /**
     * Store a newly created Coin in storage.
     *
     * @param CreateCoinRequest $request
     *
     * @return Response
     */
    public function store(CreateCoinRequest $request)
    {
        if (!Auth::user()->role->permissions->create_coin)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        try {
            Coin::create($request->all());

            return response()->json(["msg" => "Moneda guardada correctamente."], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrió un error al guardar la moneda." . $e->getMessage()], 400);
        }
    }

    /**
     * Display the specified Coin.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        
    }

    /**
     * Show the form for editing the specified Coin.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        
    }

    /**
     * Update the specified Coin in storage.
     *
     * @param  int              $id
     * @param UpdateCoinRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateCoinRequest $request)
    {
        if (!Auth::user()->role->permissions->edit_cashRegister)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        $coin = $this->coinRepository->findWithoutFail($id);

        if (empty($coin))
            return response()->json(["msg" => "No se encontró la moneda."], 404);

        try {
            $coin->update($request->all());

            return response()->json(["msg" => "Moneda actualizada correctamente."], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrió un error al actualizar la moneda." . $e->getMessage()], 400);
        }
    }

    /**
     * Remove the specified Coin from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        
    }

    /**
     * Change coin status.
     *
     * @param  int $id
     * @param  int $status
     *
     * @return Response
     */
    public function changeStatus($id, $status)
    {

        if (!Auth::user()->role->permissions->edit_cashRegister)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        $coin = Coin::find($id);
        
        if (empty($coin))
            return response()->json(["msg" => "Moneda no encontrada"], 404);

        $coin->status = $status;
        $coin->save();

        $exchangeRate = ExchangeRate::where('coin_id', $id)->activated()->first();
        
        if(!empty($exchangeRate)) {
            $exchangeRate->status = 0;
            $exchangeRate->save();
        } else {
            $er = ExchangeRate::where('coin_id', $id)->orderBy('id', 'desc')->first();
            
            if(!empty($er)) {
                $er->status = 1;
                $er->save();
            }
        }

        $textStatus = $status == 1 ? 'activada' : 'desactivada';

        return response()->json(["msg" => "Moneda " . $textStatus . " correctamente"], 200);
    }
}