<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Requests\CreateCustomerRequest;
use App\Http\Requests\UpdateCustomerRequest;
use Illuminate\Database\QueryException;
use App\Models\Customer;
use App\DataTables\CustomerDataTable;
use Auth;

class CustomerController extends Controller
{


    public function index(CustomerDataTable $customerDataTable)
    {
        return $customerDataTable->render('customers.index');
    }



    public function save(CreateCustomerRequest $request)
    {
        $data = $request->all();
        $data["rfc"] = strtoupper($data["rfc"]);
        try {

            $customer = Customer::create($data);

            return response()->json(["msg" => "Ha sido registrado correctamente", "customer" => $customer], 200);
        } catch (QueryException $e) {
            return response()->json(["msg" => "Error al registrar sus datos " . $e->getMessage()], 400);

        }

    }

    public function getInformation($rfc)
    {
        $customer = Customer::where('rfc', '=', $rfc)->where('status', 1)->first();

        if (empty($customer))
            return response()->json(["msg" => "Datos no encontrados"], 404);

        return response()->json(["customer" => $customer], 200);
    }

    public function update($id, UpdateCustomerRequest $request)
    {
        $customer = Customer::find($id);

        if (empty($customer))
            return response()->json(["msg" => "Datos no encontrados"], 404);

        $data = $request->all();
        $data["rfc"] = strtoupper($data["rfc"]);
        try {

            $customer->update($data);

            return response()->json(["msg" => "Sus datos han sido actualizados correctamente", "customer" => $customer], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Error al actualizar sus datos " . $e->getMessage()], 400);

        }

    }

    public function setStatus($id, $status)
    {

        if (!Auth::user()->role->permissions->edit_customer)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        $customer = Customer::find($id);
        if (empty($customer))
            return response()->json(["msg" => "Cliente no encontrado"], 404);

        $customer->status = $status;
        $customer->save();

        $textStatus = $status == 1 ? 'activado' : 'desactivado';
        return response()->json(["msg" => "Cliente" . $textStatus . " correctamente"], 200);

    }
}
