<?php

namespace App\Http\Controllers;

use DB;
use App\Models\Coin;
use App\Models\ExchangeRate;
use App\DataTables\ExchangeRateDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateExchangeRateRequest;
use App\Http\Requests\UpdateExchangeRateRequest;
use App\Repositories\ExchangeRateRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Response;
use Illuminate\Database\QueryException;
use Carbon\Carbon;
use Auth;

class ExchangeRateController extends AppBaseController
{
    /** @var  ExchangeRateRepository */
    private $exchangeRateRepository;

    public function __construct(ExchangeRateRepository $exchangeRateRepo)
    {
        $this->exchangeRateRepository = $exchangeRateRepo;
    }

    /**
     * Display a listing of the ExchangeRate.
     *
     * @param ExchangeRateDataTable $exchangeRateDataTable
     * @return Response
     */
    public function index(ExchangeRateDataTable $exchangeRateDataTable)
    {
        return $exchangeRateDataTable->render('exchange_rates.index', [
            "coins" => Coin::activated()->get()->pluck('description', 'id')
        ]);
    }

    /**
     * Show the form for creating a new ExchangeRate.
     *
     * @return Response
     */
    public function create()
    {
        
    }

    /**
     * Store a newly created ExchangeRate in storage.
     *
     * @param CreateExchangeRateRequest $request
     *
     * @return Response
     */
    public function store(CreateExchangeRateRequest $request)
    {
        $exchangeRate = $request->all();

        try {
            DB::beginTransaction();
            
            ExchangeRate::whereIn('coin_id', $exchangeRate['coin_id'])->update(['status' => 0]);

            $countExchangeRate = count($exchangeRate['exchange_rate']);
            for($i = 0; $i < $countExchangeRate; ++$i) {
                ExchangeRate::create([
                    'coin_id' => $exchangeRate["coin_id"][$i],
                    'exchange_rate' => $exchangeRate["exchange_rate"][$i],
                    'date' => Carbon::now()
                ]);
            }

            DB::commit();

            return response()->json(['msg' => 'Tipos de cambio actualizados correctamente.'], 200);

        } catch (QueryException $e) {
            DB::rollBack();
            return response()->json(["msg" => "Ocurrió un error al actualizar los tipos de cambio." . $e->getMessage()], 400);
        }
    }

    /**
     * Display the specified ExchangeRate.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        
    }

    /**
     * Show the form for editing the specified ExchangeRate.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        
    }

    /**
     * Update the specified ExchangeRate in storage.
     *
     * @param  int              $id
     * @param UpdateExchangeRateRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateExchangeRateRequest $request)
    {
        
    }

    /**
     * Remove the specified ExchangeRate from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        
    }

    public function getCurrent()
    {
        $types = ExchangeRate::where('coin_id', '!=', 2)->activated()->with(['coin' => function ($coin) {
            $coin->activated();
        }])->get();

        return response()->json(["exchangeRates" => $types]);
    }
}