<?php

namespace App\Http\Controllers;

use App\DataTables\FilialDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateFilialRequest;
use App\Http\Requests\UpdateFilialRequest;
use App\Repositories\FilialRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Response;
use Illuminate\Database\QueryException;
use DB;
use App\Models\Filial;
use App\Models\User;
use Auth;

class FilialController extends AppBaseController
{
    /** @var  FilialRepository */
    private $filialRepository;

    public function __construct(FilialRepository $filialRepo)
    {
        $this->filialRepository = $filialRepo;
    }

    /**
     * Display a listing of the Filial.
     *
     * @param FilialDataTable $filialDataTable
     * @return Response
     */
    public function index(FilialDataTable $filialDataTable)
    {

        return $filialDataTable->render('filials.index');
    }

    /**
     * Show the form for creating a new Filial.
     *
     * @return Response
     */
    public function create()
    {
    }

    /**
     * Store a newly created Filial in storage.
     *
     * @param CreateFilialRequest $request
     *
     * @return Response
     */
    public function store(CreateFilialRequest $request)
    {

        try {

            Filial::create($request->all());


            return response()->json(["msg" => "Sucursal registrada correctamente"], 200);

        } catch (QueryException $e) {

            return response()->json(["msg" => "Error al registrar la sucursal " . $e->getMessage()], 400);

        }

    }

    /**
     * Display the specified Filial.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $filial = $this->filialRepository->findWithoutFail($id);

        if (empty($filial)) {
            Flash::error('Filial not found');

            return redirect(route('filials.index'));
        }

        return view('filials.show')->with('filial', $filial);
    }

    /**
     * Show the form for editing the specified Filial.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $filial = Filial::find($id);
        if (empty($filial))
            return response()->json(["msg" => "Sucursal no encontrada"], 404);


        return response()->json(["filial" => $filial], 200);
    }

    /**
     * Update the specified Filial in storage.
     *
     * @param  int              $id
     * @param UpdateFilialRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateFilialRequest $request)
    {

        $filial = Filial::find($id);
        if (empty($filial))
            return response()->json(["msg" => "Sucursal no encontrada"], 404);
        try {
            $filial->update($request->all());

            return response()->json(["msg" => "Sucursal actualizada correctamente"], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Error al actualizar la sucursal " . $e->getMessage()], 400);

        }
    }

    /**
     * Remove the specified Filial from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {

    }
    public function setStatus($id, $status)
    {
        $filial = Filial::find($id);
        if (empty($filial))
            return response()->json(["msg" => "Sucursal no encontrada"], 404);

        $filial->status = $status;
        $filial->save();

        $textStatus = $status == 1 ? 'activada' : 'desactivada';
        return response()->json(["msg" => "Sucursal " . $textStatus . " correctamente"], 200);

    }

    public function getUsers($id)
    {
        $users = User::where('filial_id', '=', $id)->activated()->get();

        if (empty($users))
            return response()->json(["msg" => "Esta sucursal no tiene usuarios registrados"], 404);

        return response()->json(["users" => $users], 200);
    }
}
