<?php

namespace App\Http\Controllers;

use App\DataTables\ItemDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateItemRequest;
use App\Http\Requests\UpdateItemRequest;
use App\Repositories\ItemRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Response;
use Illuminate\Database\QueryException;
use App\Models\Item;
use Curl;
use Auth;
use App\Models\MeasureUnit;

class ItemController extends AppBaseController
{
    /** @var  ItemRepository */
    private $itemRepository;

    public function __construct(ItemRepository $itemRepo)
    {
        $this->itemRepository = $itemRepo;
    }

    /**
     * Display a listing of the Item.
     *
     * @param ItemDataTable $itemDataTable
     * @return Response
     */
    public function index(ItemDataTable $itemDataTable)
    {
        if (!Auth::user()->role->permissions->create_item && !Auth::user()->role->permissions->edit_item)
            abort(401, 'No tienes permisos para acceder a esta característica del sistema');


        return $itemDataTable->render('items.index', [
            'measureUnits' => MeasureUnit::activated()->get()->pluck('measure', 'id')
        ]);
    }

    /**
     * Show the form for creating a new Item.
     *
     * @return Response
     */
    public function create()
    {
    }

    /**
     * Store a newly created Item in storage.
     *
     * @param CreateItemRequest $request
     *
     * @return Response
     */
    public function store(CreateItemRequest $request)
    {

        try {

            Item::create($request->all());

            return response()->json(["msg" => "Producto/servicio guardado correctamente"], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrio un error al guardar el producto/servicio " . $e->getMessage()], 400);
        }
    }

    /**
     * Display the specified Item.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {

    }

    /**
     * Show the form for editing the specified Item.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $urlAPI = env('APP_ENV') == 'local' ? env("API_LOCAL") : env("API_PRODUCTION");
        $item = Item::find($id);
        if (empty($item))
            return response()->json(["msg" => "Producto/servicio no encontrado"], 404);

        $curl = Curl::to($urlAPI . '/items/satCodes/' . $item->sat_item_id . '/' . $item->sat_unit_id)
            ->asJson(true)
            ->returnResponseObject()
            ->get();

        if ($curl->status == 200)
            return response()->json([
            "item" => $item,
            "satItem" => $curl->content["satItem"],
            "satUnit" => $curl->content["satUnit"]
        ], 200);

        return response()->json(["message" => $curl->message], $curl->status);

    }

    /**
     * Update the specified Item in storage.
     *
     * @param  int              $id
     * @param UpdateItemRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateItemRequest $request)
    {
        $item = Item::find($id);

        if (empty($item))
            return response()->json(["msg" => "No se encontró el producto/servicio"], 404);


        try {

            $item->update($request->all());

            return response()->json(["msg" => "Producto/servicio actualizado correctamente"], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrio un error al actualizar el producto/servicio " . $e->getMessage()], 400);
        }

    }

    /**
     * Remove the specified Item from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {

    }

    public function setStatus($id, $status)
    {

        if (!Auth::user()->role->permissions->edit_item)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        $item = Item::find($id);
        if (empty($item))
            return response()->json(["msg" => "Producto/servicio no encontrada"], 404);

        $item->status = $status;
        $item->save();

        $textStatus = $status == 1 ? 'activado' : 'desactivado';
        return response()->json(["msg" => "Producto/servicio " . $textStatus . " correctamente"], 200);

    }
}
