<?php

namespace App\Http\Controllers;

use App\DataTables\MeasureUnitDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateMeasureUnitRequest;
use App\Http\Requests\UpdateMeasureUnitRequest;
use App\Repositories\MeasureUnitRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Response;
use App\Models\MeasureUnit;
use Auth;

class MeasureUnitController extends AppBaseController
{
    /** @var  MeasureUnitRepository */
    private $measureUnitRepository;

    public function __construct(MeasureUnitRepository $measureUnitRepo)
    {
        $this->measureUnitRepository = $measureUnitRepo;
    }

    /**
     * Display a listing of the MeasureUnit.
     *
     * @param MeasureUnitDataTable $measureUnitDataTable
     * @return Response
     */
    public function index(MeasureUnitDataTable $measureUnitDataTable)
    {
        return $measureUnitDataTable->render('measure_units.index');
    }

    /**
     * Show the form for creating a new MeasureUnit.
     *
     * @return Response
     */
    public function create()
    {
    }

    /**
     * Store a newly created MeasureUnit in storage.
     *
     * @param CreateMeasureUnitRequest $request
     *
     * @return Response
     */
    public function store(CreateMeasureUnitRequest $request)
    {
        try {

            MeasureUnit::create($request->all());

            return response()->json(["msg" => "Unidad de medida guardada correctamente"], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrio un error al guardar la unidad de medida" . $e->getMessage()], 400);
        }
    }

    /**
     * Display the specified MeasureUnit.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {

    }

    /**
     * Show the form for editing the specified MeasureUnit.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {

    }

    /**
     * Update the specified MeasureUnit in storage.
     *
     * @param  int              $id
     * @param UpdateMeasureUnitRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateMeasureUnitRequest $request)
    {
        $unit = MeasureUnit::find($id);

        if (empty($unit))
            return response()->json(["msg" => "No se encontró la unidad de medida"], 404);


        try {

            $unit->update($request->all());

            return response()->json(["msg" => "Unidad de medida actualizada correctamente"], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrio un error al actualizar la unidad de medida " . $e->getMessage()], 400);
        }
    }

    /**
     * Remove the specified MeasureUnit from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {

    }

    public function setStatus($id, $status)
    {

        if (!Auth::user()->role->permissions->edit_unit)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        $unit = MeasureUnit::find($id);
        if (empty($unit))
            return response()->json(["msg" => "Unidad de medida no encontrada"], 404);

        $unit->status = $status;
        $unit->save();

        $textStatus = $status == 1 ? 'activada' : 'desactivada';
        return response()->json(["msg" => "Unidad de medida " . $textStatus . " correctamente"], 200);

    }
}
