<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\MethodPayment;
use App\Models\ExchangeRate;
use App\Models\Coin;
use App\DataTables\MethodPaymentDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateMethodPaymentRequest;
use App\Http\Requests\UpdateMethodPaymentRequest;
use App\Repositories\MethodPaymentRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Response;
use Illuminate\Database\QueryException;

class MethodPaymentController extends AppBaseController
{
    /** @var  MethodPaymentRepository */
    private $methodPaymentRepository;

    public function __construct(MethodPaymentRepository $methodPaymentRepo)
    {
        $this->methodPaymentRepository = $methodPaymentRepo;

        // View::share('coins', Coin::activated()->pluck('description', 'id'));
    }

    /**
     * Display a listing of the MethodPayment.
     *
     * @param MethodPaymentDataTable $methodPaymentDataTable
     * @return Response
     */
    public function index(MethodPaymentDataTable $methodPaymentDataTable)
    {
        $methodPayment = [
            "tarjeta" => "TARJETA",
            "credito" => "CREDITO",
            "efectivo" => "EFECTIVO",
            "transferencia" => "TRANSFERENCIA",
            "aerolinea" => "AEROLINEA"
        ];

        return $methodPaymentDataTable->render('method_payments.index', [
            "coins" => Coin::activated()->get()->pluck('description', 'id'),
            "methodPayment" => $methodPayment
        ]);
    }

    /**
     * Show the form for creating a new MethodPayment.
     *
     * @return Response
     */
    public function create()
    {
        return view('method_payments.create');
    }

    /**
     * Store a newly created MethodPayment in storage.
     *
     * @param CreateMethodPaymentRequest $request
     *
     * @return Response
     */
    public function store(CreateMethodPaymentRequest $request)
    {
        try {
            MethodPayment::create($request->all());

            return response()->json(["msg" => "Método de pago guardado correctamente."], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrió un error al guardar el método de pago." . $e->getMessage()], 400);
        }
    }

    /**
     * Display the specified MethodPayment.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        
    }

    /**
     * Show the form for editing the specified MethodPayment.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        
    }

    /**
     * Update the specified MethodPayment in storage.
     *
     * @param  int              $id
     * @param UpdateMethodPaymentRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateMethodPaymentRequest $request)
    {
        $methodPayment = $this->methodPaymentRepository->findWithoutFail($id);

        if (empty($methodPayment))
            return response()->json(["msg" => "No se encontró el método de pago."], 404);

        try {
            $methodPayment->update($request->all());

            return response()->json(["msg" => "Método de pago actualizado correctamente."], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrió un error al actualizar el método de pago." . $e->getMessage()], 400);
        }
    }

    /**
     * Remove the specified MethodPayment from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        
    }

    /**
     * Change method payment status.
     *
     * @param  int $id
     * @param  int $status
     *
     * @return Response
     */
    public function changeStatus($id, $status)
    {
        $methodPayment = MethodPayment::find($id);
        
        if (empty($methodPayment))
            return response()->json(["msg" => "Método de pago no encontrado"], 404);

        $methodPayment->status = $status;
        $methodPayment->save();

        $textStatus = $status == 1 ? 'activado' : 'desactivado';

        return response()->json(["msg" => "Método de pago " . $textStatus . " correctamente"], 200);
    }

    public function getPayMethod(Request $request) 
    {
        $exchangeRate = $request->input('query');
        $coin_id = ExchangeRate::where('id', '=', $exchangeRate)->first();

        $methodPayment = MethodPayment::activated()
            ->where('coin_id', '=', $coin_id->coin_id)
            ->pluck('key', 'id');

        return response()->json(['methodPayment' => $methodPayment], 200);
    }
}
