<?php

namespace App\Http\Controllers;

use App\Models\Car;
use App\Models\Owner;
use App\DataTables\OwnerDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateOwnerRequest;
use App\Http\Requests\UpdateOwnerRequest;
use App\Models\Ticket;
use App\Models\TicketItem;
use App\Models\UserShift;
use App\Repositories\OwnerRepository;
use Carbon\Carbon;
use DB;
use Flash;
use App\Http\Controllers\AppBaseController;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Response;
use Illuminate\Database\QueryException;
use Illuminate\Foundation\Auth\AuthenticatesUsers;

class OwnerController extends AppBaseController
{
    /** @var  OwnerRepository */
    private $ownerRepository;

    public function __construct(OwnerRepository $ownerRepo)
    {
        $this->ownerRepository = $ownerRepo;
    }

    /**
     * Display a listing of the Owner.
     *
     * @param OwnerDataTable $ownerDataTable
     * @return Response
     */
    public function index(OwnerDataTable $ownerDataTable)
    {
        return $ownerDataTable->render('owners.index');
    }

    /**
     * Show the form for creating a new Owner.
     *
     * @return Response
     */
    public function create()
    {

    }

    /**
     * Store a newly created Owner in storage.
     *
     * @param CreateOwnerRequest $request
     *
     * @return Response
     */
    public function store(CreateOwnerRequest $request)
    {
        try {
            Owner::create($request->all());

            return response()->json(["msg" => "Dueño guardado correctamente."], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrió un error al guardar el dueño." . $e->getMessage()], 400);
        }
    }

    /**
     * Display the specified Owner.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {

    }

    /**
     * Show the form for editing the specified Owner.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        
    }

    /**
     * Update the specified Owner in storage.
     *
     * @param  int              $id
     * @param UpdateOwnerRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateOwnerRequest $request)
    {
        $owner = $this->ownerRepository->findWithoutFail($id);

        if (empty($owner))
            return response()->json(["msg" => "No se encontró el dueño."], 404);

        try {
            $owner->update($request->all());

            return response()->json(["msg" => "Dueño actualizado correctamente."], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrió un error al actualizar el dueño." . $e->getMessage()], 400);
        }
    }

    /**
     * Remove the specified Owner from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $owner = $this->ownerRepository->findWithoutFail($id);

        if (empty($owner))
            return response()->json(["msg" => "No se encontró el dueño."], 404);

        try {
            $this->ownerRepository->delete($id);

            return response()->json(["msg" => "Dueño eliminado correctamente."], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrió un error al eliminar el dueño." . $e->getMessage()], 400);
        }
    }

    /**
     * get Total charges and tickets of day Now()
     *
     * @param  int $id
     *
     * @return Response
     */
    public function getPayTicket(Request $request)
    {
        $input = $request->all();

        $owner =  $this->ownerRepository->findWithoutFail($input["id_owner"]);


        if (empty($owner))
            return response()->json(["msg" => "No se encontró el dueño."], 404);

        try {

            $cars = Car::where("owner_id","=",$input["id_owner"])->pluck('id')->toArray();

            $cashRegister = UserShift::with("tickets")->with("tickets.ticketItems")->with("tickets.ticketItems.car")
                ->with("tickets.ticketItems.car.owner")->with("user")->with("cashOutbound")->with("pay")->with("pay.ticket")
                ->where("user_id","=",Auth::user()->id)
                ->orderBy("start_shift_date","desc")
                ->where("final_shift_date","=",null)
                ->first();

            if(empty($cashRegister)){
                return response()->json(["msg" => "Esta función requiere el inicio de un Turno"], 500);
            }


            $notinvoice = $cashRegister["tickets"]->where("amount_mx","=",null)->sum("amount");
            $invoice = $cashRegister["tickets"]->where("amount_mx","!=",null)->sum("amount_mx");

            if(isset($cashRegister["cashOutbound"])) {
                $total_amount_outbound = 0;
                foreach ($cashRegister["cashOutbound"] as $outbound) {
                    $total_amount_outbound += floatval($outbound["amount"]);
                }
                $dataShift["total_amount_outbound"] = $total_amount_outbound;
            }

            if(isset($cashRegister["pay"])) {
                $total_amount_pay = 0;
                foreach ($cashRegister["pay"] as $pay) {
                    $total_amount_pay += floatval($pay["ticket"]["amount"]);
                }
                $dataShift["total_amount_pay"] = $total_amount_pay;
            }

            return response()->json(["msg" => "Todo correcto.","notInvoiced"=>$notinvoice,"invoiced"=>$invoice,"init_amount"=>$cashRegister["init_amount"],
                    "owner_data"=>$owner,"cashOutbound"=>$total_amount_outbound,"pay"=>$total_amount_pay], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrió un error al obtneer información del servidor." . $e->getMessage()], 400);
        }
    }

    /**
     * get Login View
     *
     * @return VIew
     */
    public function showLogin(){
        return view('user_owner.login');
    }

    /**
     * POST Login action
     *
     * @return Response
     */
    public function login(Request $request){
        dd($request->all());

    }


}
