<?php

namespace App\Http\Controllers;

use App\Models\Importar\Rates;
use Illuminate\Http\Request;
use DB;
use App\Models\Rate;
use App\DataTables\RateDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateRateRequest;
use App\Http\Requests\UpdateRateRequest;
use App\Repositories\RateRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Response;
use Illuminate\Database\QueryException;

class RateController extends AppBaseController
{
    /** @var  RateRepository */
    private $rateRepository;

    public function __construct(RateRepository $rateRepo)
    {
        $this->rateRepository = $rateRepo;
    }

    /**
     * Display a listing of the Rate.
     *
     * @param RateDataTable $rateDataTable
     * @return Response
     */
    public function index(RateDataTable $rateDataTable)
    {
        return $rateDataTable->render('rates.index');
    }

    /**
     * Show the form for creating a new Rate.
     *
     * @return Response
     */
    public function create()
    {
        
    }

    /**
     * Store a newly created Rate in storage.
     *
     * @param CreateRateRequest $request
     *
     * @return Response
     */
    public function store(CreateRateRequest $request)
    {
        try {
            $input = $request->all();

            Rate::create([
                'destination' => $input['destination'],
                'locality' => empty($input['locality']) ? '' : $input['locality'],
                'cost' => $input['cost'],
                'status' => $input['status'],
                'measure_unit_id' => !isset($input['measure_unit_id']) ? '2' : $input['measure_unit_id'],
                'sat_item_id' => !isset($input['sat_item_id']) ? '46407' : $input['sat_item_id'],
                'sat_unit_id' => !isset($input['sat_unit_id']) ? '1615' : $input['sat_unit_id'],
            ]);


            return response()->json(["msg" => "Tarifa guardada correctamente."], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrió un error al guardar la tarifa." . $e->getMessage()], 400);
        }
    }

    /**
     * Display the specified Rate.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        
    }

    /**
     * Show the form for editing the specified Rate.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        
    }

    /**
     * Update the specified Rate in storage.
     *
     * @param  int              $id
     * @param UpdateRateRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateRateRequest $request)
    {
        $rate = Rate::find($id);

        if (empty($rate))
            return response()->json(["msg" => "No se encontró la tarifa."], 404);

        try {
            $rate->update($request->all());

            return response()->json(["msg" => "Tarifa actualizada correctamente."], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrió un error al actualizar la tarifa." . $e->getMessage()], 400);
        }
    }

    /**
     * Remove the specified Rate from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $rate = $this->rateRepository->findWithoutFail($id);

        if (empty($rate))
            return response()->json(["msg" => "No se encontró la tarifa."], 404);

        try {
            $this->rateRepository->delete($id);

            return response()->json(["msg" => "Tarifa eliminada correctamente."], 200);

        } catch (QueryException $e) {
            return response()->json(["msg" => "Ocurrió un error al eliminar la tarifa." . $e->getMessage()], 400);
        }
    }
    /**
     * Get rates for autofill.
     *
     * @return Response
     */
    public function getRates(Request $request)
    {
        $rates = Rate::select('id', 'destination', 'locality', DB::raw('CONCAT(destination, " ", locality, " : $", cost) as rates'))
            ->where('destination', 'like', $request->input('query') . '%')
            ->orWhere('locality', 'like', $request->input('query') . '%')
            ->pluck('rates', 'id');

        return response()->json(['rates' => $rates], 200);
    }
    
     function updloadNewRates(Request $request){
        try {
            /*Rate::query()->delete();*/

            if ($request->hasFile('rates')) {
                $file = $request->file('rates');
                if ($file->getMimeType() == 'application/vnd.ms-office' || $file->getMimeType() == 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet') {
                    if (!is_dir(storage_path() . '/app/importaciones/rates'))
                        Storage::makeDirectory('importaciones/rates/');
                    $path = 'importaciones/rates/' . time() . '.' . $file->getClientOriginalExtension();
                    Storage::put($path, file_get_contents($file));
                    $result = Rates::newRates($path);
                    return response()->json($result, 200);
                } else {
                    return response()->json(['error' => 'Ha intentado subir un archivo no válido, únicamente se permiten archivos Excel.'], 400);
                }
            } else {
                return response()->json(['error' => 'Es necesario adjuntar el archivo con la información de los destinos'], 400);
            }


        }catch(\Exception $e){
            Log::info($e);
        }


    }
    
}
