<?php

namespace App\Http\Controllers;

use App\DataTables\RoleDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateRoleRequest;
use App\Http\Requests\UpdateRoleRequest;
use App\Http\Controllers\AppBaseController;
use App\Models\Role;
use Response;
use DB;
use Auth;


class RoleController extends AppBaseController
{
    /** @var  RoleRepository */

    public function __construct()
    {
    }

    /**
     * Display a listing of the Role.
     *
     * @param RoleDataTable $roleDataTable
     * @return Response
     */
    public function index(RoleDataTable $roleDataTable)
    {

        if (!Auth::user()->role->permissions->create_role && !Auth::user()->role->permissions->edit_role)
            abort(401, 'No tienes permisos para acceder a esta característica');

        return $roleDataTable->render('roles.index');
    }

    /**
     * Show the form for creating a new Role.
     *
     * @return Response
     */
    public function create()
    {
    }

    /**
     * Store a newly created Role in storage.
     *
     * @param CreateRoleRequest $request
     *
     * @return Response
     */
    public function store(CreateRoleRequest $request)
    {

        if (!Auth::user()->role->permissions->create_role)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica del sistema'], 401);

        try {

            DB::beginTransaction();


            $role = Role::create($request->get('role'));
            $role->permissions()->create($request->get('role_permission'));

            DB::commit();

            return response()->json(["msg" => "Rol registrado correctamente"], 200);

        } catch (QueryException $e) {

            DB::RolelBack();

            return response()->json(["msg" => "Error al registrar el rol " . $e->getMessage()], 400);

        }

    }

    /**
     * Display the specified Role.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {

    }

    /**
     * Show the form for editing the specified Role.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {

        if (!Auth::user()->role->permissions->edit_role)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica del sistema'], 401);

        $role = Role::with(['permissions'])->find($id);
        if (empty($role))
            return response()->json(["msg" => "Rol no encontrado"], 404);


        return response()->json(["role" => $role], 200);

    }

    /**
     * Update the specified Role in storage.
     *
     * @param  int $id
     * @param UpdateRoleRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateRoleRequest $request)
    {


        if (!Auth::user()->role->permissions->edit_role)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica del sistema'], 401);

        $role = Role::find($id);

        if (empty($role))
            return response()->json(["msg" => "Rol no encontrado"], 404);

        try {

            DB::beginTransaction();

            $role->update($request->get("role"));
            $role->permissions()->update($request->get('role_permission'));

            DB::commit();

            return response()->json(["msg" => "Rol actualizado correctamente"], 200);

        } catch (QueryException $e) {

            DB::RolelBack();

            return response()->json(["msg" => "Error al actualizar el rol " . $e->getMessage()], 400);

        }

    }

    /**
     * Remove the specified Role from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {

    }

    public function setStatus($id, $status)
    {

        if (!Auth::user()->role->permissions->edit_role)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica del sistema'], 401);

        $role = Role::find($id);
        if (empty($role))
            return response()->json(["msg" => "Rol no encontrado"], 404);

        $role->status = $status;
        $role->save();

        $textStatus = $status == 1 ? 'activado' : 'desactivado';
        return response()->json(["msg" => "Rol " . $textStatus . " correctamente"], 200);
    }
}
