<?php

namespace App\Http\Controllers;

use App\DataTables\TaxConfigDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateTaxConfigRequest;
use App\Http\Requests\UpdateTaxConfigRequest;
use App\Http\Controllers\AppBaseController;
use Response;
use App\Http\Requests\CreateTaxItemConfigRequest;
use Illuminate\Database\QueryException;
use DB;
use App\Models\TaxConfig;
use Auth;
use Curl;

class TaxConfigController extends AppBaseController
{

    public function __construct()
    {

    }

    /**
     * Display a listing of the TaxConfig.
     *
     * @param TaxConfigDataTable $taxConfigDataTable
     * @return Response
     */
    public function index(TaxConfigDataTable $taxConfigDataTable)
    {
        return $taxConfigDataTable->render('tax_configs.index');
    }

    /**
     * Show the form for creating a new TaxConfig.
     *
     * @return Response
     */
    public function create()
    {

    }

    /**
     * Store a newly created TaxConfig in storage.
     *
     * @param CreateTaxConfigRequest $request
     *
     * @return Response
     */
    public function store(CreateTaxConfigRequest $request, CreateTaxItemConfigRequest $requestTaxes)
    {

        try {

            DB::beginTransaction();

            $taxConfig = TaxConfig::create($request->input("tax_config"));
            $numTaxes = count($request->input('tax_item_config')["tax_id"]);
            for ($i = 0; $i < $numTaxes; ++$i) {
                $taxConfig->taxItemConfigs()->create([
                    'tax_id' => $request->input('tax_item_config')["tax_id"][$i]
                ]);
            }

            DB::commit();

            return response()->json(["msg" => "Configuración guardada correctamente"], 200);

        } catch (QueryException $e) {
            DB::rollBack();
            return response()->json(["msg" => "Error al guardar la configuración de impuestos " . $e->getMessage()], 400);
        }

    }

    /**
     * Display the specified TaxConfig.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {

    }

    /**
     * Show the form for editing the specified TaxConfig.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {

        $urlAPI = env('APP_ENV') == 'local' ? env("API_LOCAL") : env("API_PRODUCTION");
        $taxConfig = TaxConfig::find($id);
        if (empty($taxConfig))
            return response()->json(["msg" => "Configuración de impuestos no encontrada"], 404);

        $curl = Curl::to($urlAPI . '/taxes/fetchFederals')
            ->withData(["taxes" => $taxConfig->taxItemConfigs()->select('tax_id')->get()->pluck('tax_id')->toArray()])
            ->asJson(true)
            ->returnResponseObject()
            ->post();

        if ($curl->status == 200)
            return response()->json(["taxConfig" => $taxConfig, "taxes" => $curl->content], 200);

        return response()->json(["message" => $curl->message], $curl->status);
    }

    /**
     * Update the specified TaxConfig in storage.
     *
     * @param  int              $id
     * @param UpdateTaxConfigRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateTaxConfigRequest $request)
    {

        $taxConfig = TaxConfig::find($id);
        if (empty($taxConfig))
            return response()->json(["msg" => "Configuración de impuestos no encontrada"], 404);



        try {

            DB::beginTransaction();

            $taxConfig->update([
                'name' => $request["tax_config"]["name"]
            ]);

            $numTaxes = count($request->input('tax_item_config')["tax_id"]);
            for ($i = 0; $i < $numTaxes; ++$i) {
                $taxConfig->taxItemConfigs()->firstOrCreate([
                    'tax_id' => $request->input('tax_item_config')["tax_id"][$i]
                ]);
            }

            DB::commit();

            return response()->json(["msg" => "Configuración actualizada correctamente"], 200);

        } catch (QueryException $e) {
            DB::rollBack();
            return response()->json(["msg" => "Error al actualizar la configuración de impuestos " . $e->getMessage()], 400);
        }

    }

    /**
     * Remove the specified TaxConfig from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {

    }

    public function setStatus($id, $status)
    {

        if (!Auth::user()->role->permissions->edit_tax)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        $taxConfig = TaxConfig::find($id);
        if (empty($taxConfig))
            return response()->json(["msg" => "Configuración de impuestos no encontrada"], 404);

        $taxConfig->status = $status;
        $taxConfig->save();

        $textStatus = $status == 1 ? 'activada' : 'desactivada';
        return response()->json(["msg" => "Configuración de impuestos " . $textStatus . " correctamente"], 200);

    }


}
