<?php

namespace App\Http\Controllers;

use App\Models\Coin;
use App\Models\Export\EXCEL\ExportTicket;
use App\Models\MethodPayment;
use App\Models\Owner;
use App\Models\UserShift;
use App\Models\Utilities;
use DB;
use View;
use Carbon\Carbon;
use App\Models\Item;
use App\Models\Ticket;
use App\Models\TaxConfig;
use Illuminate\Http\Request;
use App\Models\Invoice\InvoiceXml;
use Illuminate\Support\Facades\Auth;
use App\Http\Requests\CreateTicketRequest;
use Illuminate\Database\QueryException;
use Illuminate\Support\Facades\Log;
use Mail;
use App\Mail\SendTicket;
use App\Models\SoapCalls;
use Mike42\Escpos\CapabilityProfile;
use Mike42\Escpos\PrintConnectors\WindowsPrintConnector;
use Storage;
use URL;
use App\Models\Customer;
use Curl;
use App\DataTables\TicketDataTable;
use App\Mail\SendInvoice;
use App\DataTables\InvoiceHistoryDataTable;
use App\Models\Filial;
use App\Models\Car;
use App\Models\ExchangeRate;
use App\Models\Rate;
use Jenssegers\Date\Date;
use App\Models\Invoice\GlobalInvoiceXml;
use App\Models\Ticket\TicketPrint;
use App\Models\User;
use Mike42\Escpos\PrintConnectors\FilePrintConnector;
use Mike42\Escpos\Printer;


class TicketController extends Controller
{
    /**
     * TicketController constructor.
     */
    public function __construct()
    {
        $this->middleware('owner', ['only' => 'index']);

        View::share('owners', Owner::get()->pluck("name","id"));
        View::share('method_payments', MethodPayment::with("coin")->select("key","id")->where("status",1)->pluck("key","id"));
        View::share('coins', Coin::get()->pluck("key","id"));
    }

    public function index(TicketDataTable $ticketDataTable)
    {

        return $ticketDataTable->render(
            'tickets.index',
            [
                "customers" => Customer::get()->pluck('name', 'id'),
                "filials" => Filial::activated()->get()->pluck('name', 'id')
            ]
        );
    }


    public function sell()
    {
        $urlAPI = env('APP_ENV') == 'local' ? env("API_LOCAL") : env("API_PRODUCTION");


        $payMethod = Curl::to($urlAPI . '/payMethods/fetch')
            ->asJson(true)
            ->returnResponseObject()
            ->get();


        /*if ($payMethod->status != 200) {
            abort(400, "Ocurrió un error intente más tarde");
        }else{

        }*/


        $activedShift = UserShift::where("user_id","=",Auth::user()->id)
            ->orderBy("start_shift_date","desc")
            ->where("final_shift_date","=",null)->first();

        $exchangeRateNow = ExchangeRate::join('coin', 'coin.id', '=', 'exchange_rate.coin_id')
            ->where('exchange_rate.status', '=', 1)
            ->where(DB::raw("DATE_FORMAT(exchange_rate.date, '%Y%m%d')"),"=",Carbon::now()->format("Ymd"))
            ->get();



        return view('tickets.sell')
            ->with('filials', Filial::activated()->pluck('name', 'id'))
            ->with('cars', Car::select(DB::raw('CONCAT(car.name, " : ", owner.name) AS carOwner'), DB::raw('car.id AS carId'))
                ->join('owner', 'owner.id', '=', 'car.owner_id')
                ->orderBy('carId', 'asc')
                ->get()
                ->pluck('carOwner', 'carId'))
            ->with('exchangeRates', ExchangeRate::select(DB::raw('CONCAT(coin.key, " : ", coin.description, " $", exchange_rate.exchange_rate) AS exchangeRateName'), DB::raw('exchange_rate.id AS exchangeRateId'))
                ->join('coin', 'coin.id', '=', 'exchange_rate.coin_id')
                ->where('exchange_rate.status', '=', 1)
                ->get()
                ->pluck('exchangeRateName', 'exchangeRateId'))
            ->with('payMethods', collect($payMethod->content["payMethods"])->pluck("descripcion", "clave"))
            ->with('coins',Coin::pluck("key","id"))
            ->with("items", Item::activated()->get())
            ->with("activeShift",$activedShift)
            ->with("taxConfigs", TaxConfig::activated()->get()->pluck("name", "id"));

    }

    public function store(CreateTicketRequest $request)
    {


        $user = User::find(Auth::user()->id);

        $shift = UserShift::where("user_id","=",$user["id"])
                            ->where("final_shift_date","=",null)
                            ->orderBy("start_shift_date","desc")->first();

        $exchangeRate = ExchangeRate::where("id","=",$request["exchange_rate_id"])
                        ->where(DB::raw("DATE_FORMAT(date, '%Y%m%d')"),"=",Carbon::now()->format("Ymd"))
                        ->first();
        Log::info(json_encode($user['id']));
        Log::info(json_encode($shift));
        Log::info(json_encode($exchangeRate));
    
        /*return response()->json(["msg" => "Hubo un error al generar el ticket, intente nuevamente",
                                "data" => $exchangeRate], 400);*/
        try {

            $numPayMethod = count($request["pay_method"]["method_payment_id"]);

            $random_string = Utilities::generate_string(2);
            $random_string2 = Utilities::generate_string(2);

            $month_string = Carbon::now()->format("m");
            $day_string = Carbon::now()->format("d");
            $time_string = Carbon::now()->format("Hi");

            DB::beginTransaction();
            $ticket = Ticket::create([
                'shift_user_id' => $shift["id"],
                'customer_id' => Customer::where('rfc', 'XAXX010101000')->first()->id,
                'job_order' => null,
                'number_ticket' => null,
                'invoice' => null,
                'paymethod' => $request["ticket"]["paymethod"],
                'date' => Carbon::now()->toDateTimeString(),
                'date_bond' => null,
                'subtotal' => $request["ticket"]["subtotal"],
                'subtotal_mx' => $request["ticket"]["exchange_rate_id"] == 1 ? $request["ticket"]["subtotal"] : $request["ticket"]["subtotal_mx"],
                'iva' => $request["ticket"]["iva"],
                'iva_mx' => $request["ticket"]["exchange_rate_id"] == 1 ? $request["ticket"]["subtotal"] : $request["ticket"]["iva_mx"],
                'ret_iva' => null,
                'amount' => $request["ticket"]["total"],
                'amount_mx' => $request["ticket"]["exchange_rate_id"] == 1 ? $request["ticket"]["subtotal"] : $request["ticket"]["total_mx"],
                'note' => $request['ticket']['note']
            ]);

            $numItems = count($request["ticket_item"]["price"]);

            $desc_method_payment = "";
            for ($i = 0; $i < $numPayMethod; ++$i) {
                $methodPayment = MethodPayment::find($request["pay_method"]["method_payment_id"][$i]);

                $ticket->ticketMethodPayment()->create([
                    'method_payment_id' => $request["pay_method"]["method_payment_id"][$i],
                    'exchange_rate_id' => $request["ticket"]["exchange_rate_id"],
                    'amount' => $request["pay_method"]["quantity"][$i],
                ]);

                $desc_method_payment .= "($".$request["pay_method"]["quantity"][$i]." ".$methodPayment["coin"]["key"]." ".strtoupper ($methodPayment["key"]). " )\n";
            }

            for ($i = 0; $i < $numItems; ++$i) {


                $descripcion = "Auto: " . $request["ticket_item"]["car"][$i] . ", Destino: " . $request["ticket_item"]["rate"][$i] .
                            "\n Forma de Pago: " . $desc_method_payment;

                $ticket->ticketItems()->create([
                    'car_id' => $request["ticket_item"]["car_id"][$i],
                    'item_id' => $request["ticket_item"]["rate_id"][$i],
                    'quantity' => 1,
                    'description' => $descripcion,
                    'price' => $request["ticket_item"]["exchangeRatePrice"][$i],
                    'price_mx' => $request["ticket_item"]["price"][$i],
                    'iva' => $request["ticket_item"]["exchangeRateIva"][$i],
                    'iva_mx' => $request["ticket_item"]["iva"][$i],
                    'amount' => $request["ticket_item"]["exchangeRateSubtotal"][$i],
                    'amount_mx' => $request["ticket_item"]["subtotal"][$i],
                    'discount' => $request["ticket_item"]["exchangeRateDiscount"][$i],
                    'discount_mx' => $request["ticket_item"]["discount"][$i],
                ]);
            }

            $request["randomFolio"] = $ticket->number_ticket;

            if($user->filial["related_filial"] !== null){
                $this->filial_invoice = Filial::find($user->filial["related_filial"]);
            }else {
                $this->filial_invoice = $user->filial;
            }


            $txtTickets = new InvoiceXml($request, $ticket, $user);
            if (!$txtTickets->generateXml()) {
                DB::rollBack();
                return response()->json(["msg" => "Hubo un error al generar el ticket, intente nuevamente"], 400);

            }

            $var = Ticket::where("id","=",$ticket->id)
                ->with("userShift")->with("userShift.user")->with("customer")
                ->with("ticketItems")->with("ticketMethodPayment")
                ->with("ticketMethodPayment.methodPayment")
                ->with("ticketMethodPayment.exchangeRate")->first();

            if(isset($request["ticket"]["withFactura"])) {
                if ($request["ticket"]["withFactura"] == "on") {

                    $ticket = Ticket::where("id", "=", $var["id"])
                        ->with("userShift")->with("userShift.user")->with("customer")
                        ->with("ticketItems")->with("ticketMethodPayment")->first();

                    if (empty($ticket))
                        return response()->json(["msg" => "No se encontró el ticket"], 404);

                    $customer = Customer::find($request["customer_id"]);
                    if (empty($customer))
                        return response()->json(["msg" => "No se encontraron los datos del cliente"], 404);

                    if ($ticket->status == 'invoiced')
                        return response()->json(["msg" => "Este ticket ya ha sido facturado"], 400);

                    $user = User::find(Auth::user()->id);
                    $txtTickets = new InvoiceXml(["invoice"=>"invoice"], $var, $user);

                    if (!$txtTickets->generateXml() ) {
                        return response()->json(["msg" => "Hubo un error al obtener datos del ticket, intente nuevamente"], 400);
                    }

                    $ticket->update([
                        "invoice" => $this->filial_invoice->prefix.$this->filial_invoice->number_ticket
                    ]);

                    $this->filial_invoice->update([
                        'number_ticket' => $this->filial_invoice->number_ticket + 1
                    ]);

                    DB::commit();
                    DB::beginTransaction();

                    //Update XML
                    $path = app_path() . '/../storage/app/filials/' . $this->filial_invoice->id . '/tickets/' .$ticket->number_ticket. '.xml';
                
                    $xml = new \DOMDocument();
                    $xml->loadXML(file_get_contents($path));
                    $doc = $xml->documentElement;
                    $receptor = $doc->getElementsByTagName("Receptor");
                    $receptor[0]->setAttribute("Rfc", $customer->rfc);
                    $receptor[0]->setAttribute("Nombre", $customer->name);
                    $xml->save($path);


                    //Invoice signing
                    $response = SoapCalls::getInvoice($path);

                    if (empty($response["xml"]))
                        return response()->json(["msg" => "Error al timbrar la factura " . $response["message"]], 400);

                    //Update ticket's total
                    $file = Storage::get('filials/' . $this->filial_invoice->id . '/tickets/' .$ticket->number_ticket . '.xml');
                    //$file = Storage::get('filials/' . $user->filial_id . '/tickets/' . $ticket->number_ticket . '_' . $customer->type . '.xml');

                    $curl = SoapCalls::readCfdi($file);
                    if ($curl->status != 200)
                        return response()->json([
                            "msg" => "Factura timbrada correctamente, ocurrió un error al guardar los totales",
                        ], 400);

                    if ($customer->type == 'moral') {
                        $ticket->subtotal = $curl->content["data"]["factura"]["subtotal"];
                        $ticket->iva = $curl->content["data"]["impuestos_federales"]["totalTraslados"];
                        $ticket->ret_iva = $curl->content["data"]["impuestos_federales"]["totalRetenciones"];
                    }

                    //Delete tickets
                    Storage::delete('filials/' . $this->filial_invoice->id . '/tickets/' . $ticket->number_ticket . '.xml');

                    //Save invoice files
                    Storage::put('filials/' . $this->filial_invoice->id . '/facturas/' . $ticket->number_ticket. '.xml', base64_decode($response["xml"]));
                    Storage::put('filials/' . $this->filial_invoice->id . '/facturas/' . $ticket->number_ticket. '.pdf', base64_decode($response["pdf"]));

                    $ticket->status = 'invoiced';
                    $ticket->customer_id = $request["customer_id"];
                    $ticket->ret_iva = null;
                    $ticket->save();


                    $basepath = storage_path() . '/app/filials/' . $this->filial_invoice->id . '/facturas/';

                    try {

                        Mail::to($customer->email)
                        ->send(new SendInvoice($basepath . $ticket->number_ticket . '.xml', $basepath . $ticket->number_ticket . '.pdf', $ticket, []));

                    } catch (\Swift_TransportException $e) {
                        DB::commit();
                        return response()->json([
                            "msg" => "Factura timbrada, ocurrió un error al enviar al correo proporcionado",
                            "file" => URL::to('/') . '/../storage/app/filials/' . $this->filial_invoice->id  . '/facturas/' . $ticket->number_ticket . '.pdf',
                            "ticket" => $var, "msg" => "Venta almacenada correctamente",
                            "body" => view('tickets.ticketToModal')->with('ticket', $var)->render(),
                            "src" => url('tickets/print/' . $ticket->id),
                        ], 200);
                    }

                    DB::commit();

                    return response()->json([
                        "msg" => "Factura timbrada y enviada a su correo correctamente",
                        "file" => URL::to('/') . '/../storage/app/filials/' . $this->filial_invoice->id  . '/facturas/' .$ticket->number_ticket . '.pdf',
                        "ticket" => $var, "msg" => "Venta almacenada correctamente",
                        "body" => view('tickets.ticketToModal')->with('ticket', $var)->render(),
                        "src" => url('tickets/print/' . $ticket->id),
                    ], 200);
                }
            }

            //$print = TicketPrint::printTicket($var);
            DB::commit();
            return response()->json([
                "ticket" => $var, "msg" => "Venta almacenada correctamente",
                "body" => view('tickets.ticketToModal')->with('ticket', $var)->render(),
                "src" => url('tickets/print/' . $ticket->id)
            ], 200);


        }catch (QueryException $e) {
            DB::rollBack();
            return response()->json(["msg" => "Error al guardar la venta " . $e->getMessage()], 400);
        }

        // $user = empty($request["ticket"]["filial_id"]) ? Auth::user() :
        //     User::where('filial_id', $request["ticket"]["filial_id"])->first();

        // try {
        //     DB::beginTransaction();
        //     $ticket = Ticket::create([
        //         'user_id' => $user->id,
        //         'job_order' => $request['ticket']['job_order'],
        //         'subtotal' => $request["physical_ticket"]["subtotal"],
        //         'iva' => $request["physical_ticket"]["iva"],
        //         'ret_iva' => null,
        //         'customer_id' => Customer::where('rfc', 'XAXX010101000')->first()->id,
        //         'date' => Carbon::now()->toDateTimeString(),
        //         'date_limit' => Carbon::now()->addMonth(1)->day(10),
        //         'number_ticket' => $user->filial->prefix . '' . $user->filial->number_ticket,
        //         'amount' => array_sum($request["ticket_item"]["amount"]),
        //         'note' => $request['ticket']['note']
        //     ]);

        //     $numItems = count($request["ticket_item"]["price"]);
        //     for ($i = 0; $i < $numItems; ++$i) {
        //         $ticket->ticketItems()->create([
        //             'item_id' => $request["ticket_item"]["item_id"][$i],
        //             'quantity' => $request["ticket_item"]["quantity"][$i],
        //             'description' => $request["ticket_item"]["description"][$i],
        //             'price' => $request["ticket_item"]["price"][$i],
        //             'amount' => $request["ticket_item"]["amount"][$i]
        //         ]);
        //     }

        //     $txtTickets = new InvoiceXml($request, $ticket, $user);

        //     if (!$txtTickets->generateXml('physical') || !$txtTickets->generateXml('moral')) {
        //         DB::rollBack();
        //         return response()->json(["msg" => "Hubo un error al generar el ticket, intente nuevamente"], 400);

        //     }

        //     $user->filial()->update([
        //         'number_ticket' => $user->filial->number_ticket + 1
        //     ]);

        //     DB::commit();

        //     if (!empty($request["ticket"]["email"])) {
        //         $copies = !empty($request["copies"]) ? $request["copies"] : [];

        //         Mail::to($request["ticket"]["email"], $copies)
        //             ->send(new SendTicket($ticket, $copies));
        //     }

        //     return response()->json(["ticket" => $ticket, "msg" => "Venta almacenada correctamente"], 200);

        // } catch (QueryException $e) {
        //     DB::rollBack();
        //     return response()->json(["msg" => "Error al guardar la venta " . $e->getMessage()], 400);
        // } catch (\Swift_TransportException $e) {
        //     return response()->json(["msg" => "El ticket se generó correctamente, hubo un error al enviarlo por correo " . $e->getMessage()], 200);

        // }
    }

    public function printPaper($id)
    {
        return view('tickets.ticket')
            ->with('ticket', Ticket::find($id));

    }

    public function printPaper2($id)
    {
        return response()->json([
            "msg" => "Ticket cargado correctamente",
            "body" => view('tickets.ticketToModal')->with('ticket', Ticket::where("id","=",$id)
                ->with("userShift")->with("userShift.user")->with("customer")
                ->with("ticketItems")->with("ticketMethodPayment")
                ->with("ticketMethodPayment.methodPayment")
                ->with("ticketMethodPayment.exchangeRate")->first()
                )
                ->render()
        ], 200);
    }

    public function getTotals(Request $request){

        $from = $request->get('txtFrom');
        $to = $request->get('txtTo');
        $status = $request->get('status');
        $customer = $request->get('customer');
        $filial = $request->get('filial');
        $user = $request->get('user');
        $owner = $request->get('owner');
        $personType = $request->get('person_type');
        $withPayed = $request->get('withPayed');
        $withInvoice = $request->get('withInvoice');
        $withMoney = $request->get('withMoney');
        $noValid = $request->get('noValid');

        $dashboard =  Ticket::query()->select(
            "ticket.*",
            DB::raw("IF(pay = 1 OR status = 'cancelled',0,amount) AS saldo"))
            ->with("ticketItems")->with("ticketItems.car")->with("ticketItems.car.owner")
            ->with("ticketMethodPayment")->with("ticketMethodPayment.methodPayment")->with("ticketMethodPayment.methodPayment.coin")
            ->with("ticketMethodPayment.exchangeRate")->with("ticketMethodPayment.exchangeRate.coin")
            ->with("userShift")->with("userShift.user")
            ->with("customer")->with("ticketItems");

        $dashboard->whereHas("ticketItems.car", function ($query) use ($owner) {
            if (!empty($owner)) {
                $query->where('car.owner_id', '=', $owner);
            }
        });

        if (!Auth::user()->role->permissions->general_report)
            $dashboard->where('user_shift.user_id', '=', Auth::user()->id);
        if (!empty($user))
            $dashboard->where('user_shift.user_id', '=', $user);
        if (!empty($personType))
            $dashboard->where('customer.type', '=', $personType);
        if (!empty($filial)){
            $dashboard->where('user_shift.user.filial_id', '=', $filial);
        }
        if (!empty($customer)){
            $dashboard->where('customer_id', '=', $customer);
        }
        if (!empty($from) && !empty($to)){
            $dashboard->whereBetween(DB::raw('DATE_FORMAT(date,"%Y-%m-%d")'), [$from, $to]);
        }
        if (!empty($status)){
            $dashboard->whereIn('ticket.status', $status);
        }
        if (!empty($withInvoice) || !empty($noValid) || !empty($withPayed)){
            if($withInvoice == 1){
                $dashboard->where('ticket.status', "LIKE", "%invoice%");
            }
            if($noValid == 1){
                $dashboard->where('ticket.status', "=" , "system_no_valid");
            }
            if($withPayed == 1){
                $dashboard->where('ticket.pay', $withPayed);
            }
        }
        if (!empty($withMoney)){
            $dashboard
                ->withCount(["ticketMethodPayment AS total_efectivo" => function ($q) use($withMoney) {
                    if($withMoney == 1){
                        $q->where("method_payment_id", "=", 2);
                    }
                }])
                ->whereHas("ticketMethodPayment.methodPayment", function ($query) use($withMoney){
                    if($withMoney == 1){
                        $query->where("type","=", "efectivo");
                    }
                });
        }

        $dashboardData["amountTotal"]= 0;
        $dashboardData["noTicket"]= 0;
        $dashboardData["noTicketInvalid"]= 0;
        $dashboardData["amountInvoice"]= 0;
        $dashboardData["amountInvoiceCancelled"]= 0;
        $dashboardData["amountPay"]= 0;
        $dashboardData["amountNoValid"]= 0;
        $dashboardData["amountMoney"]= [];

        foreach($dashboard->get()->toArray() as $t => $ticket){
            if ($ticket["status"] == "invoice") {
                $dashboardData["amountInvoice"] += floatval($ticket["amount_mx"]);
            }elseif ($ticket["status"] == "invoice_cancelled" ) {
                $dashboardData["amountInvoiceCancelled"] += floatval($ticket["amount_mx"]);
            } elseif ($ticket["status"] == "system_no_valid" ) {
                $dashboardData["amountNoValid"] += floatval($ticket["amount_mx"]);
                $dashboardData["noTicketInvalid"]++;
            }

            if ($ticket["pay"]) {
                $dashboardData["amountPay"] += floatval($ticket["amount_mx"]);
            }

            foreach($ticket["ticket_method_payment"] as $tmp => $methodPayment) {
                if($methodPayment["method_payment"]["type"] == "efectivo") {

                    $key = $methodPayment["exchange_rate"]["coin_id"];
                    if(!isset($dashboardData["amountMoney"][$key]["monto"])) {
                        $dashboardData["amountMoney"][$key]["monto"] = floatval($methodPayment["amount"]);
                        $dashboardData["amountMoney"][$key]["key"] = $methodPayment["exchange_rate"]["coin"]["key"];
                    }else{
                        $dashboardData["amountMoney"][$key]["monto"] += floatval($methodPayment["amount"]);
                        $dashboardData["amountMoney"][$key]["key"] = $methodPayment["exchange_rate"]["coin"]["key"];
                    }
                }

                if ($methodPayment["exchange_rate_id"] != 1) {
                    $dashboardData["amountTotal"] += floatval($ticket["ticket_items"]["amount_mx"]);
                } else {
                    $dashboardData["amountTotal"] += floatval($methodPayment["amount"]);

                }

            }
            $dashboardData["noTicket"]++;
        }

        return response()->json(array("message"=>"Cálculo realizado correctamente","dashboard" => $dashboardData ));
    }

    public function getTicketInformation($numberTicket, $customer_type)
    {

        $ticket = Ticket::where('number_ticket', '=', strtoupper($numberTicket))
            ->with("ticketItems")
            ->first();
        if (empty($ticket))
            return response()->json(["msg" => "Ticket no encontrado"], 404);

        //Check if ticket is available
        if ($ticket->status != 'sell')
            return response()->json(["msg" => "El ticket ya ha sido facturado"], 400);

        $user = User::where("id","=",$ticket->userShift->user_id)->first();
        $file = "";

        if($user->filial["related_filial"] !== null){
            $this->filial_invoice = Filial::find($user->filial["related_filial"]);
        }else {
            $this->filial_invoice = $user->filial;
        }

        if(empty($ticket["invoice"])){
            if (is_file(storage_path('app/filials/' . $this->filial_invoice->id . '/tickets/' . $ticket->number_ticket . '_' . $customer_type . '.xml'))) {
                $file = Storage::get('filials/' . $this->filial_invoice->id . '/tickets/' . $ticket->number_ticket . '_' . $customer_type . '.xml');
            } else if (is_file(storage_path('app/filials/' . $this->filial_invoice->id . '/tickets/' . $ticket->number_ticket . '.xml'))) {
                $file = Storage::get('filials/' . $this->filial_invoice->id . '/tickets/' . $ticket->number_ticket . '.xml');
            } else {
                $reponse = $this->ticketXMLRemake($ticket["id"]);
                if($reponse)
                    $file = Storage::get('filials/' . $this->filial_invoice->id . '/tickets/' . $ticket->number_ticket . '.xml');
            }
        }else{
            $file = Storage::get('filials/' . $this->filial_invoice->id . '/tickets/' . $ticket->invoice . '.xml');
        }


        //Read CFDI via API
        $curl = SoapCalls::readCfdi($file);
        if ($curl->status == 200)
            return response()->json([
            "ticket" => $ticket,
            "items" => $curl->content,
        ], 200);


        return response()->json(["msg" => "Hubo un error al leer la información del ticket"], 400);


    }

    public function invoice()
    {
        $urlAPI = env('APP_ENV') == 'local' ? env("API_LOCAL") : env("API_PRODUCTION");
        $contract = Curl::to($urlAPI . '/payMethods/fetch')
            ->asJson(true)
            ->returnResponseObject()
            ->get();


        return view('tickets.invoice.ticket');
    }

    public function invoiceSigning(Request $request)
    {

        $ticket = Ticket::where("id","=",$request["ticket_id"])
            ->with("userShift")->with("userShift.user")->with("customer")
            ->with("ticketItems")->with("ticketMethodPayment")->first();
        //$ticket = Ticket::find($request["ticket_id"]);

        Log::info(json_encode($ticket));
        if (empty($ticket))
            return response()->json(["msg" => "No se encontró el ticket"], 404);

        $customer = Customer::find($request["customer_id"]);
        if (empty($customer))
            return response()->json(["msg" => "No se encontraron los datos del cliente"], 404);

        if ($ticket->status == 'invoiced')
            return response()->json(["msg" => "Este ticket ya ha sido facturado"], 400);


        $user = User::where("id","=",$ticket->userShift->user_id)->first();
        
        if($user->filial["related_filial"] !== null){
            $this->filial = $user->filial;
            $this->filial_invoice = Filial::find($user->filial["related_filial"]);
        }else {
            $this->filial = $user->filial;
            $this->filial_invoice = $user->filial;
        }
        Log::info(json_encode($this->filial_invoice));
        $txtTickets = new InvoiceXml(["invoice"=>"invoice"], $ticket, $user);

        if (!$txtTickets->generateXml() ) {
            return response()->json(["msg" => "Hubo un error al obtener datos del ticket, intente nuevamente"], 400);
        }

        if($ticket["invoice"] === null) {
            $ticket->update([
                "invoice" => $this->filial_invoice->prefix . $this->filial_invoice->number_ticket
            ]);

            $this->filial_invoice->update([
                'number_ticket' => $this->filial_invoice->number_ticket + 1
            ]);
        }


        //Update XML
        $path = app_path() . '/../storage/app/filials/' . $this->filial_invoice->id . '/tickets/' . $ticket->invoice . '.xml';
        $xml = new \DOMDocument();
        $xml->loadXML(file_get_contents($path));
        $doc = $xml->documentElement;
        $receptor = $doc->getElementsByTagName("Receptor");
        $receptor[0]->setAttribute("Rfc", $customer->rfc);
        $receptor[0]->setAttribute("Nombre", $customer->name);
        $xml->save($path);


        Log::info(json_encode($customer));
        Log::info(json_encode($ticket));
        //Invoice signing
        $response = SoapCalls::getInvoice($path);

        if (empty($response["xml"]))
            return response()->json(["msg" => "Error al timbrar la factura " . $response["message"]], 400);

        //Update ticket's total
        $file = Storage::get('filials/' . $this->filial_invoice->id . '/tickets/' .$ticket->invoice . '.xml');
        $curl = SoapCalls::readCfdi($file);
        if ($curl->status != 200)
            return response()->json([
            "msg" => "Factura timbrada correctamente, ocurrió un error al guardar los totales",
        ], 400);

        if ($customer->type == 'moral') {
            $ticket->subtotal = $curl->content["data"]["factura"]["subtotal"];
            $ticket->iva = $curl->content["data"]["impuestos_federales"]["totalTraslados"];
            $ticket->ret_iva = ($curl->content["data"]["impuestos_federales"]["totalRetenciones"] == null)? 0:$curl->content["data"]["impuestos_federales"]["totalRetenciones"];
        }

        //Delete tickets
        Storage::delete('filials/' . $this->filial_invoice->id . '/tickets/' .$ticket->invoice . '.xml');


        //Save invoice files
        Storage::put('filials/' . $this->filial_invoice->id  . '/facturas/' . $ticket->invoice. '.xml', base64_decode($response["xml"]));
        Storage::put('filials/' . $this->filial_invoice->id  . '/facturas/' . $ticket->invoice. '.pdf', base64_decode($response["pdf"]));

        $ticket->status = 'invoiced';
        $ticket->customer_id = $request["customer_id"];
        $ticket->save();


        $basepath = storage_path() . '/app/filials/' . $this->filial_invoice->id . '/facturas/';

        try {


            Mail::to($customer->email)
                ->send(new SendInvoice($basepath . $ticket->invoice . '.xml', $basepath . $ticket->invoice. '.pdf', $ticket, ["depapp@isotech.mx"]));

            return response()->json([
                "msg" => "Factura timbrada y enviada correctamente",
                "file" => URL::to('/') . '/../storage/app/filials/' . $this->filial_invoice->id . '/facturas/' . $ticket->invoice . '.pdf'
            ], 200);

        } catch (\Swift_TransportException $e) {

            return response()->json([
                "msg" => "Factura timbrada, ocurrió un error al enviar al correo proporcionado",
                "file" => URL::to('/') . '/../storage/app/filials/' . $this->filial_invoice->id  . '/facturas/' . $ticket->invoice . '.pdf'
            ], 200);
        }
    }

    public function sendInvoice(Request $request)
    {
        $type = $request["type"] == 'invoice' ? 'Factura' : 'Ticket';
        $invoice = Ticket::where("id","=",$request->input('id'))
        ->with("userShift")->with("userShift.user")->with("customer")
        ->with("ticketItems")->with("ticketMethodPayment")->first();
        $copies = empty($request->get("copies")) ? [] : $request->get("copies");


        if (empty($invoice))
            return response()->json(["msg" => "Factura no encontrada"], 404);


        try {

            switch ($type) {
                case 'Factura':
                    if($invoice->userShift->user->filial["related_filial"] !== null){
                        $filial = Filial::find($invoice->userShift->user->filial["related_filial"]);
                    }else{
                        $filial = $invoice->userShift->user->filial;
                    }

                    $basepath = storage_path() . '/app/filials/' . $filial->id . '/facturas/';
                    Mail::to($request->input("email"))
                        ->send(new SendInvoice($basepath . $invoice->invoice . '.xml', $basepath . $invoice->invoice . '.pdf', $invoice, $copies));

                    return response()->json(["msg" => "Factura enviada correctamente"], 200);

                    break;

                case 'Ticket':
                    Mail::to($request["email"])
                        ->send(new SendTicket($invoice, $copies));

                    return response()->json(["msg" => "Ticket enviado correctamente"], 200);

                    break;
            }


        } catch (\Swift_TransportException $e) {
            return response()->json(["msg" => "Error al enviar la factura " . $e->getMessage()], 400);

        }

    }


    public function cancelTicket($id)
    {
        $ticket = Ticket::find($id);


        if (empty($ticket))
            return response()->json(["msg" => "Factura no encontrada"], 404);

        $ticket->status = 'cancelled';
        $ticket->save();

        return response()->json(["msg" => "Ticket cancelado correctamente"], 200);


    }

    public function requestCancelTicket(Request $request)
    {
        $ticket = Ticket::find($request["id_invoice"]);

        if (empty($ticket))
            return response()->json(["msg" => "Factura no encontrada"], 404);

        $ticket->status = 'request_cancelled';
        $ticket->note = $request->comment;
        $ticket->save();

        return response()->json(["msg" => "Petición de Cancelación enviada"], 200);

    }

    public function invalidTicket($id)
    {
        $ticket = Ticket::find($id);


        if (empty($ticket))
            return response()->json(["msg" => "Ticket no encontrado"], 404);

        if($ticket->status != "system_no_valid") {
            $ticket->status = 'system_no_valid';
        }else{
            $ticket->status = 'sell';
        }

        $ticket->save();

        return response()->json(["msg" => "Ticket Invalidado correctamente"], 200);


    }

    public function cancelInvoice($id)
    {


        $invoice = Ticket::where("id","=",$id)
            ->with("userShift")->with("userShift.user")->with("customer")
            ->with("ticketItems")->with("ticketMethodPayment")->first();


        if (empty($invoice))
            return response()->json(["msg" => "Factura no encontrada"], 404);

        $response = SoapCalls::cancelInvoice($invoice->invoice);

        if (!$response["success"])
            return response()->json(["msg" => "Error al cancelar la factura " . $response["message"]], 400);


        if($invoice->userShift->user->filial["related_filial"] !== null){
            $filial = Filial::find($invoice->userShift->user->filial["related_filial"]);
        }else{
            $filial = $invoice->userShift->user->filial;
        }
        //Save invoice files
        Storage::put('filials/' . $filial->id . '/facturas/' . $invoice->invoice . '_acuse.xml', base64_decode($response["xmlAcuse"]));
        Storage::put('filials/' . $filial->id . '/facturas/' . $invoice->invoice . '_acuse.pdf', base64_decode($response["pdfAcuse"]));
        Storage::delete('filials/' . $filial->id . '/facturas/' . $invoice->invoice . '.pdf');
        Storage::put('filials/' . $filial->id . '/facturas/' . $invoice->invoice . '.pdf', base64_decode($response["pdfCancelado"]));


        $invoice->status = 'invoice_cancelled';
        $invoice->save();


        return response()->json(["msg" => "Factura cancelada correctamente"], 200);


    }

    public function invoiceHistory(InvoiceHistoryDataTable $dataTable)
    {
        return $dataTable->render('tickets.historyTicket.index');

    }

    public function editNote(Request $request)
    {

        $ticket = Ticket::find($request["ticket_id"]);
        if (empty($ticket))
            return response()->json(['msg' => 'Ticket no encontrado'], 404);


        try {
            $ticket->note = $request["note"];
            $ticket->save();

            return response()->json(['msg' => 'Observaciones actualizadas correctamente'], 200);

        } catch (QueryException $e) {
            return response()->json(['msg' => 'Error al actualizar las observaciones, intente mas tarde'], 500);

        }




    }

    public function setBond(Request $request)
    {

        if (!$request["invoice"])
            return response()->json(["msg" => "Debe anexar al menos 1 factura para el registro de depósito"], 400);

        try {

            Ticket::whereIn('id', $request['invoice'])
                ->update(['date_bond' => $request['date']]);

            return response()->json(['msg' => 'Depósito guardado correctamente'], 200);
        } catch (QueryException $e) {
            return response()->json(['msg' => 'Error al guardar el depósito, intente nuevamente'], 400);

        }


    }

    public function globalInvoice()
    {
        $urlAPI = env('APP_ENV') == 'local' ? env("API_LOCAL") : env("API_PRODUCTION");

        if (!Auth::user()->role->permissions->sign_global)
            abort(401, 'No tiene acceso a esta característica del sistema');

        $payMethod = Curl::to($urlAPI . '/payMethods/fetch')
            ->asJson(true)
            ->returnResponseObject()
            ->get();


        if ($payMethod->status != 200)
            abort(400, "Ocurrió un error intente más tarde");

        return view('tickets.globalInvoice')
            ->with('payMethods', collect($payMethod->content["payMethods"])->pluck("descripcion", "clave"));
    }

    public function getGlobalTickets($filial, $from, $to ,$methodPayment = null)
    {

        $tickets = Ticket::select(
            'ticket.id',
            'ticket.shift_user_id',
            DB::raw('DATE_FORMAT(date,"%d/%m%/%Y") as dateF'),
            'number_ticket',
            'invoice',
            'ticket.subtotal_mx',
            'ticket.iva_mx',
            'ticket.iva',
            'ticket.amount_mx'
        )

            ->has("userShift")
            ->whereHas("userShift.user", function($query) use($filial){
                $query->where('filial_id', $filial);
            })
            ->with("ticketMethodPayment.methodPayment")
            ->with("userShift")
            ->with("userShift.user")
            ->with("userShift.user.filial")
            ->whereHas("ticketMethodPayment.methodPayment", function ($query) use ($methodPayment) {
                if(empty($methodPayment)) {
                    $query->where('type',"=", "efectivo");
                }else {
                    $query->where('id',"=", $methodPayment);
                }
            })
            ->where('ticket.status','sell')
            ->whereBetween('date', [$from, $to])
            ->orderBy('subtotal', 'desc')
            ->get()->toArray();

        foreach ($tickets as $index => $ticket) {
            $ticket["subtotal_mx"] = round($ticket["amount_mx"] /*/ 1.16*/, 4);
            $ticket["iva_mx"] = round($ticket["iva"] /** 0.16*/, 4);

            //Validar que efectivamente no sea un ticket, al revisar en la carpeta de facturas.
            if($ticket["user_shift"]["user"]["filial"]["related_filial"] !== null){
                $filial_route = Filial::find($ticket["user_shift"]["user"]["filial"]["related_filial"]);
            }else{
                $filial_route = Filial::find($filial);
            }

            $path = storage_path() . '/app/filials/' . $filial_route->id . '/facturas/' . $ticket["invoice"] . '.xml';
            if (is_file($path)) {
                unset($tickets[$index]);
            }
        }

        return response()->json(['tickets' => $tickets,"id_method"=>$methodPayment], 200);

    }

    public function signGlobalInvoice(Request $request)
    {

        try {
            DB::beginTransaction();
            //dd($request->all());
            if(Auth::user()->filial["related_filial"] !== null){
                $filial = Filial::find(Auth::user()->filial["related_filial"]);
                $filial_ticket = Filial::find(Auth::user()->filial_id);
            }else{
                $filial = Filial::find(Auth::user()->filial_id);
            }


            $ids = [];
            $noTickets = count($request["ticket"]["items"]["sku"]);

            for ($i = 0; $i < $noTickets; ++$i) {
                Storage::delete('filials/' . $filial_ticket->id . '/tickets/' . $request["ticket"]["items"]["sku"][$i] . '.xml');
                $ids[] = $request["ticket"]["items"]["id"][$i];
            }

            $shift = UserShift::where("final_shift_date","=",null)
                ->orderBy("start_shift_date","desc")->first();

            Ticket::whereIn('id', $ids)
                ->update(['status' => 'global_sell']);

            $invoice = Ticket::create([
                'shift_user_id' => $shift["id"],
                'customer_id' => Customer::where('rfc', 'XAXX010101000')->first()->id,
                'invoice' => $filial->prefix . $filial->number_ticket,
                'number_ticket' => null,
                'date' => Carbon::now()->toDateTimeString(),
                'subtotal' => $request["ticket"]["subtotal"],
                'iva' => $request["ticket"]["iva"],
                'amount' => $request["ticket"]["amount"],
                'status' => 'invoice_sell'
            ]);


            $xmlFile = GlobalInvoiceXml::createXml($filial, $request->all());

            if (!$xmlFile["is_file"])
                return response()->json(["msg" => "Ocurrió un error al generar la factura, intente nuevamente[1]"], 400);

            //Insert related tickets
            foreach ($ids as $id) {
                $invoice->ticketGlobalItems()->create([
                    'ticket_related_id' => $id
                ]);
            }

            $invoice->customer_id = $invoice["customer_id"];
            $invoice->save();

            $filial->number_ticket = $filial->number_ticket + 1;
            $filial->save();

            DB::commit();


            $signin = SoapCalls::getInvoice($xmlFile["path"]);
            if (empty($signin["xml"]))
                return response()->json(["msg" => "Ocurrió un error al generar la factura, intente nuevamente[2]" . $signin["message"]], 400);


            //Storage::delete('filials/' . $filial->id . '/tickets/' . $filial->prefix . $filial->number_ticket . '.xml');
            Storage::put('filials/' . $filial->id . '/facturas/' . $filial->prefix . $filial->number_ticket . '.xml', base64_decode($signin["xml"]));
            Storage::put('filials/' . $filial->id . '/facturas/' . $filial->prefix . $filial->number_ticket . '.pdf', base64_decode($signin["pdf"]));

            /*Update tickets status to know they were included on the invoice, update filial sequential
              and insert the global invoice
             */

            return response()->json([
                "msg" => "Factura timbrada correctamente",
                "file" => URL::to('/') . '/../storage/app/filials/' . $filial->id . '/facturas/' . $invoice->number_ticket . '.pdf'
            ], 200);


        } catch (QueryException $e) {
            DB::rollBack();
            return response()->json(["msg" => "Ocurrió un error al guardar la factura en BD, la factura se timbro !!! " . $e->getMessage()], 400);
        }
    }



    public function getSearchTicket(Request $request){

        $input = $request->all();

        $arrayTickets = array();

        if(isset($input["ticketCharged"])) {
            $arrayTickets = $input["ticketCharged"];

            $assocArray = array();
            foreach ($arrayTickets as $key => $ticketID) {
                $assocArray[$key] = true;
            }
            array_diff_assoc($arrayTickets, $assocArray);
        }

        $ticket = Ticket::whereHas("ticketItems",function ($query) use($input){
                        $query->whereHas("car",function ($subquery) use($input){
                            $subquery->where("owner_id","=",$input["id_owner"]);
                        });
                    })
                    ->with("ticketItems")->with("ticketItems.car")->with("ticketItems.item")
                    ->where("number_ticket","LIKE","%".$input["ticket"]."%")
                    ->where("pay","=",null)
                    ->whereNotIn('id', $arrayTickets)
                    ->get()->toArray();

        return response()->json(array("success"=>1,"tickets"=>$ticket));
    }

    public function ticketXMLRemake($id){



        $ticket = Ticket::where("id","=",$id)
            ->with("userShift")->with("userShift.user")->with("customer")
            ->with("ticketItems")->with("ticketMethodPayment")->first();

        $user = User::where("id","=",$ticket->userShift->user_id)->first();

        $txtTickets = new InvoiceXml(["remake"=>"remake"], $ticket, $user);

        if (!$txtTickets->generateXml()) {
            return response()->json(["msg" => "Hubo un error al generar el ticket, intente nuevamente"], 400);
        }
        return response()->json(["msg" => "XML re-generado correctamente "], 200);
    }

    public function ticketXMLNew($id){


        $ticket = Ticket::where("id","=",$id)
            ->with("userShift")->with("userShift.user")->with("customer")
            ->with("ticketItems")->with("ticketMethodPayment")->first();

        $user = User::where("id","=",$ticket->userShift->user_id)->first();

        $txtTickets = new InvoiceXml([], $ticket, $user);

        if (!$txtTickets->generateXml()) {
            return response()->json(["msg" => "Hubo un error al generar el ticket, intente nuevamente"], 400);
        }
        return response()->json(["msg" => "XML re-generado correctamente "], 200);
    }

    public function ticketRemakeAll(){
        set_time_limit ( 1500);

        $tickets = Ticket::where("status","=","sell")->get();

        $array_errors = [];

        foreach($tickets as $ticket_change) {

            $ticket = Ticket::where("id", "=", $ticket_change["id"])
                ->with("userShift")->with("userShift.user")->with("customer")
                ->with("ticketItems")->with("ticketMethodPayment")->first();

            $user = User::where("id","=",$ticket->userShift->user_id)->first();

            $txtTickets = new InvoiceXml([], $ticket, $user);

            if (!$txtTickets->generateXml()) {
                $array_errors[$ticket_change["id"]] = $ticket_change;
                //return response()->json(["msg" => "Hubo un error al generar el ticket, intente nuevamente"], 400);
            }
        }
        return response()->json(["msg" => "XML re-generado correctamente ","errors",$array_errors], 200);

    }


    public function printExample(){
        $connector = new WindowsPrintConnector("EPSON TM-T20II Receipt");
        //$connector = new FilePrintConnector("php://stdout");
        $printer = new Printer($connector);
        $printer -> cut();
        $printer -> close();
    }

    public function reInvoice($id)
    {

        $ticket = Ticket::where("id","=",$id)
            ->with("userShift")->with("userShift.user")->with("customer")
            ->with("ticketItems")->with("ticketItems.car")->with("ticketItems.item")
            ->with("ticketMethodPayment")
            ->with("ticketMethodPayment.methodPayment")
            ->with("ticketMethodPayment.exchangeRate")->first();


        $user = User::where("id","=",$ticket->userShift->user_id)->first();

        $customer = Customer::where("id","=",$ticket["customer_id"])->first();


        $shift = UserShift::where("user_id","=",$user["id"])
            ->where("final_shift_date","=",null)
            ->orderBy("start_shift_date","desc")->first();

        try {
            $numPayMethod = count($ticket["ticketMethodPayment"]);

            $random_string = Utilities::generate_string(2);
            $random_string2 = Utilities::generate_string(2);

            $month_string = Carbon::now()->format("m");
            $day_string = Carbon::now()->format("d");
            $time_string = Carbon::now()->format("Hi");

            DB::beginTransaction();
            $ticket_nuevo = Ticket::create([
                'shift_user_id' => $ticket["shift_user_id"],
                'customer_id' => $ticket["customer_id"],
                'job_order' => null,
                'number_ticket' => null,
                'invoice' => null,
                'paymethod' => null,
                'date' => Carbon::now()->toDateTimeString(),
                'date_bond' => null,
                'subtotal' => $ticket["subtotal"],
                'subtotal_mx' => $ticket["subtotal_mx"],
                'iva' => $ticket["iva"],
                'iva_mx' => $ticket["iva_mx"],
                'ret_iva' => null,
                'amount' => $ticket["amount"],
                'amount_mx' => $ticket["amount_mx"],
                'note' => $ticket["note"],
            ]);

            $numItems = count($ticket["ticketItems"]["price"]);

            $desc_method_payment = "";
            for ($i = 0; $i < $numPayMethod; ++$i) {

                $methodPayment = MethodPayment::find($ticket["ticketMethodPayment"][$i]["methodPayment"]["id"]);
                $exchangeRate = ExchangeRate::find($ticket["ticketMethodPayment"][$i]["exchangeRate"]["id"]);

                $ticket_nuevo->ticketMethodPayment()->create([
                    'method_payment_id' => $methodPayment["id"],
                    'exchange_rate_id' => $exchangeRate["id"],
                    'amount' => $ticket["ticketMethodPayment"][$i]["amount"],
                ]);

                $desc_method_payment .= "($".$ticket["ticketMethodPayment"][$i]["amount"]." ".$methodPayment["coin"]["key"]." ".strtoupper ($methodPayment["key"]). " )\n";
            }


            for ($i = 0; $i < $numItems; ++$i) {

                $descripcion = "(".$ticket["number_ticket"]. ")" . "Auto: " . $ticket["ticketItems"]["car"]["name"] . ", Destino: " . $ticket["ticketItems"]["item"]["destination"] .
                    "\n Forma de Pago: " . $desc_method_payment;


                $ticket_nuevo->ticketItems()->create([
                    'car_id' => $ticket["ticketItems"]["car"]["id"],
                    'item_id' => $ticket["ticketItems"]["item"]["id"],
                    'quantity' => 1,
                    'description' => $descripcion,
                    'price' => $ticket["ticketItems"]["price"],
                    'price_mx' => $ticket["ticketItems"]["price_mx"],
                    'iva' => $ticket["ticketItems"]["iva"],
                    'iva_mx' => $ticket["ticketItems"]["iva_mx"],
                    'amount' => $ticket["ticketItems"]["amount"],
                    'amount_mx' => $ticket["ticketItems"]["amount_mx"],
                    'discount' => $ticket["ticketItems"]["discount"],
                    'discount_mx' => $ticket["ticketItems"]["discount_mx"],
                ]);
            }

            $txtTickets = new InvoiceXml(array("reinvoice"=>"reinvoice","ticket_reinvoice"=> $ticket["number_ticket"] ), $ticket_nuevo, $user);

            if (!$txtTickets->generateXml($ticket["type"])) {
                DB::rollBack();
                return response()->json(["msg" => "Hubo un error al generar el ticket, intente nuevamente"], 400);

            }


            $ticket_nuevo->update([
                "invoice" => $user->filial->filial_invoice->prefix.$user->filial->filial_invoice->number_ticket
            ]);

            $user->filial->filial_invoice->update([
                'number_ticket' => $user->filial->filial_invoice->number_ticket + 1
            ]);

            DB::commit();

                //Update XML
                $path = app_path() . '/../storage/app/filials/' . $ticket_nuevo->userShift->user->filial_id . '/tickets/' . $ticket_nuevo->number_ticket . '_' . $customer->type . '.xml';
                $xml = new \DOMDocument();
                $xml->loadXML(file_get_contents($path));
                $doc = $xml->documentElement;
                $receptor = $doc->getElementsByTagName("Receptor");
                $receptor[0]->setAttribute("Rfc", $customer->rfc);
                $receptor[0]->setAttribute("Nombre", $customer->name);
                $xml->save($path);




                    //Invoice signing
                    $response = SoapCalls::getInvoice($path);

                    if (empty($response["xml"]))
                        return response()->json(["msg" => "Error al timbrar la factura " . $response["message"]], 400);

                    //Update ticket's total
                    $file = Storage::get('filials/' . $ticket_nuevo->userShift->user->filial->id. '/tickets/' . $ticket_nuevo->number_ticket . '_' . $customer->type . '.xml');
                    $curl = SoapCalls::readCfdi($file);
                    if ($curl->status != 200)
                        return response()->json([
                            "msg" => "Factura timbrada correctamente, ocurrió un error al guardar los totales",
                        ], 400);

                    if ($customer->type == 'moral') {

                        $ticket_nuevo->subtotal = $curl->content["data"]["factura"]["subtotal"];
                        $ticket_nuevo->iva = $curl->content["data"]["impuestos_federales"]["totalTraslados"];
                        $ticket_nuevo->ret_iva = ($curl->content["data"]["impuestos_federales"]["totalRetenciones"] == "")?0:$curl->content["data"]["impuestos_federales"]["totalRetenciones"];
                    }

                    //Delete tickets
                    Storage::delete('filials/' . $ticket_nuevo->userShift->user->filial_id . '/tickets/' . $ticket_nuevo->number_ticket . '_fisica.xml');
                    Storage::delete('filials/' . $ticket_nuevo->userShift->user->filial_id . '/tickets/' . $ticket_nuevo->number_ticket . '_moral.xml');

                    //Save invoice files
                    Storage::put('filials/' . $ticket_nuevo->userShift->user->filial->related_filial . '/facturas/' . $ticket_nuevo->invoice . '.xml', base64_decode($response["xml"]));
                    Storage::put('filials/' . $ticket_nuevo->userShift->user->filial->related_filial . '/facturas/' . $ticket_nuevo->invoice . '.pdf', base64_decode($response["pdf"]));

                    $ticket_nuevo->status = 'invoiced';
                    $ticket_nuevo->customer_id = !empty($request)? $request["customer_id"]: $ticket["customer_id"];
                    $ticket_nuevo->save();


                    $basepath = storage_path() . '/app/filials/' . $ticket_nuevo->userShift->user->filial->related_filial . '/facturas/';

                    try {

                        Mail::to($ticket_nuevo->customer->email)
                            ->send(new SendInvoice($basepath .$ticket_nuevo->invoice . '.xml', $basepath . $ticket_nuevo->invoice . '.pdf', $ticket_nuevo, []));

                    } catch (\Swift_TransportException $e) {

                        return response()->json([
                            "msg" => "Factura timbrada, ocurrió un error al enviar al correo proporcionado",
                            "file" => URL::to('/') . '/../storage/app/filials/' . $ticket_nuevo->userShift->user->filial->related_filial . '/facturas/' . $ticket_nuevo->invoice . '.pdf',
                            "ticket" => $ticket_nuevo, "msg" => "Venta almacenada correctamente",
                            "body" => view('tickets.ticketToModal')->with('ticket', $ticket_nuevo)->render(),
                            "src" => url('tickets/print/' . $ticket->id),
                        ], 200);
                    }

                    return response()->json([
                        "msg" => "Factura timbrada y enviada correctamente",
                        "file" => URL::to('/') . '/../storage/app/filials/' . $ticket_nuevo->userShift->user->filial->related_filial . '/facturas/' . $ticket_nuevo->invoice . '.pdf'
                    ], 200);


            /*    }
            }*/

            //$print = TicketPrint::printTicket($var);

            /*return response()->json([
                "ticket" => $var, "msg" => "Venta almacenada correctamente",
                "body" => view('tickets.ticketToModal')->with('ticket', $var)->render(),
                "src" => url('tickets/print/' . $ticket_nuevo->id)
            ], 200);*/


        }catch (QueryException $e) {
            DB::rollBack();
            return response()->json(["msg" => "Error al guardar la venta " . $e->getMessage()], 400);
        }

    }

    public function report(Request $request){
        ini_set('max_execution_time', 1500);
        $from = $request->get('txtFrom');
        $to = $request->get('txtTo');
        $status = $request->get('status');
        $customer = $request->get('customer');
        $filial = $request->get('filial');
        $user = $request->get('user');
        $owner =$request->get('owners');
        $coin =$request->get('coin');
        $methodPayment =$request->get('methodPayment');
        $personType =$request->get('person_type');
        $withPayed =$request->get('withPayed');
        $withInvoice =$request->get('withInvoice');
        $withMoney =$request->get('withMoney');
        $noValid =$request->get('noValid');

        $invoices = Ticket::query()->select(
            "ticket.*",
            DB::raw("IF(pay = 1 OR status = 'cancelled',0,amount) AS saldo"))
            ->with("ticketItems")->with("ticketItems.car")->with("ticketItems.car.owner")
            ->with("ticketMethodPayment")->with("ticketMethodPayment.methodPayment")->with("ticketMethodPayment.methodPayment.coin")
            ->with("ticketMethodPayment.exchangeRate")->with("ticketMethodPayment.exchangeRate.coin")
            ->with("userShift")->with("userShift.user")
            ->with("customer")->with("ticketItems");

        $invoices->whereHas("ticketItems.car", function ($query) use ($owner) {
            if (!empty($owner)) {
                $query->where('car.owner_id', '=', $owner);
            }
        });

        if (!Auth::user()->role->permissions->general_report)
            $invoices->where('user_shift.user_id', '=', Auth::user()->id);
        if (!empty($user))
            $invoices->where('user_shift.user_id', '=', $user);
        if (!empty($personType))
            $invoices->where('customer.type', '=', $personType);
        if (!empty($filial)){
            $invoices->where('user_shift.user.filial_id', '=', $filial);
        }
        if (!empty($customer)){
            $invoices->where('customer_id', '=', $customer);
        }
        if (!empty($from) && !empty($to)){
            $invoices->whereBetween(DB::raw('DATE_FORMAT(date,"%Y-%m-%d")'), [$from, $to]);
        }
        if (!empty($status)){
            $invoices->whereIn('ticket.status', $status);
        }
        if (!empty($withPayed)){
            if($withPayed == 1){
                $invoices->where('ticket.pay', $withPayed);
            }
        }
        if (!empty($withInvoice)){
            if($withInvoice == 1){
                $invoices->where('ticket.status', "LIKE", "%invoice");
            }
        }
        if (!empty($noValid)){
            if($noValid == 1){
                $invoices->where('ticket.status', "=" , "system_no_valid");
            }
        }else{
            $invoices->where('ticket.status', "!=" , "system_no_valid");
        }
        if (!empty($withMoney)){
            $invoices
                ->withCount(["ticketMethodPayment AS total_efectivo" => function ($q) use($withMoney) {
                    if($withMoney == 1){
                        $q->where("method_payment_id", "=", 2);
                    }
                }])
                ->whereHas("ticketMethodPayment.methodPayment", function ($query) use($withMoney){
                    if($withMoney == 1){
                        $query->where("type","=", "efectivo");
                    }
                });
        }

        $data = $invoices->get()->toArray();

        $report = ExportTicket::ticketReport($data,$from,$to,$withMoney);

        return response()->json([
            "success" => 1,
            "file" => URL::to('/'). '/../storage/exports' . '/app/reporte/' . $report['file'],
            "msg" => "Correcto"
        ]);
    }

}
