<?php

namespace App\Http\Controllers;

use App\DataTables\TicketPayDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateTicketPayRequest;
use App\Http\Requests\UpdateTicketPayRequest;
use App\Models\Ticket;
use App\Models\TicketPay;
use App\Models\UserShift;
use App\Repositories\TicketPayRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Response;

class TicketPayController extends AppBaseController
{
    /** @var  TicketPayRepository */
    private $ticketPayRepository;

    public function __construct(TicketPayRepository $ticketPayRepo)
    {
        $this->ticketPayRepository = $ticketPayRepo;
    }

    /**
     * Display a listing of the TicketPay.
     *
     * @param TicketPayDataTable $ticketPayDataTable
     * @return Response
     */
    public function index(TicketPayDataTable $ticketPayDataTable)
    {
        return $ticketPayDataTable->render('ticket_pays.index');
    }

    /**
     * Show the form for creating a new TicketPay.
     *
     * @return Response
     */
    public function create()
    {
        return view('ticket_pays.create');
    }

    /**
     * Store a newly created TicketPay in storage.
     *
     * @param CreateTicketPayRequest $request
     *
     * @return Response
     */
    public function store(CreateTicketPayRequest $request)
    {
        $input = $request->all();

        $dataShift = UserShift::with("user")
            ->where("user_id","=",Auth::user()->id)
            ->orderBy("start_shift_date","desc")
            ->where("final_shift_date","=",null)
            ->first();

        if(isset($input["ticketCharged"])){

            foreach ($input["ticketCharged"] as $ticket){
                TicketPay::create([
                    "user_shift_id" => $dataShift["id"],
                    "ticket_id" => $ticket,
                ]);

                $ticketFind = Ticket::find($ticket);

                $ticketFind->pay = 1;
                $ticketFind->save();
            }
        }

        return response()->json(array("success"=>1,"msg"=> "Pagos creados correctamente"));
    }


    public function storeTicketPay($id_ticket)
    {
        $dataShift = UserShift::with("user")
            ->where("user_id","=",Auth::user()->id)
            ->orderBy("start_shift_date","desc")
            ->where("final_shift_date","=",null)
            ->first();
        $ticketFind = Ticket::find($id_ticket);

        if(!isset($dataShift))
            return response()->json(array("success"=>0,"msg"=> "No hay turno abierto"));
        if(!isset($ticketFind))
            return response()->json(array("success"=>0,"msg"=> "No se encontró ticket"));

        $ticketPay = TicketPay::create([
            "user_shift_id" => $dataShift["id"],
            "ticket_id" => $id_ticket,
        ]);

        $ticketFind->pay = 1;
        $ticketFind->save();

        return response()->json(array("success"=>1,"msg"=> "Pago creado correctamente"));
    }

    /**
     * Display the specified TicketPay.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $ticketPay = $this->ticketPayRepository->findWithoutFail($id);

        if (empty($ticketPay)) {
            Flash::error('Ticket Pay not found');

            return redirect(route('ticketPays.index'));
        }

        return view('ticket_pays.show')->with('ticketPay', $ticketPay);
    }

    /**
     * Show the form for editing the specified TicketPay.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $ticketPay = $this->ticketPayRepository->findWithoutFail($id);

        if (empty($ticketPay)) {
            Flash::error('Ticket Pay not found');

            return redirect(route('ticketPays.index'));
        }

        return view('ticket_pays.edit')->with('ticketPay', $ticketPay);
    }

    /**
     * Update the specified TicketPay in storage.
     *
     * @param  int              $id
     * @param UpdateTicketPayRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateTicketPayRequest $request)
    {
        $ticketPay = $this->ticketPayRepository->findWithoutFail($id);

        if (empty($ticketPay)) {
            Flash::error('Ticket Pay not found');

            return redirect(route('ticketPays.index'));
        }

        $ticketPay = $this->ticketPayRepository->update($request->all(), $id);

        Flash::success('Ticket Pay updated successfully.');

        return redirect(route('ticketPays.index'));
    }

    /**
     * Remove the specified TicketPay from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $ticketPay = $this->ticketPayRepository->findWithoutFail($id);

        if (empty($ticketPay)) {
            Flash::error('Ticket Pay not found');

            return redirect(route('ticketPays.index'));
        }

        $this->ticketPayRepository->delete($id);

        Flash::success('Ticket Pay deleted successfully.');

        return redirect(route('ticketPays.index'));
    }
}
