<?php

namespace App\Http\Controllers;

use App\DataTables\UserDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateUserRequest;
use App\Http\Requests\UpdateUserRequest;
use App\Http\Requests\ChangePasswordRequest;
use App\Mail\UserCreated;
use App\Models\Role;
use App\Models\Filial;
use App\Models\User;
use App\Http\Controllers\AppBaseController;
use Illuminate\Database\QueryException;
use Response;
use Mail;
use View;
use Auth;
use Request;
use Illuminate\Support\Facades\Hash;

class UserController extends AppBaseController
{

    public $client;


    public function __construct()
    {

        View::share('filials', Filial::activated()->pluck('name', 'id'));
        View::share('roles', Role::activated()->pluck('name', 'id'));
    }

    /**
     * Display a listing of the UsuarioSucursal.
     *
     * @param UserDataTable $userDataTable
     * @return Response
     */
    public function index(UserDataTable $userDataTable)
    {

        if (!Auth::user()->role->permissions->create_user && !Auth::user()->role->permissions->edit_user)
            abort(401, 'No tienes acceso a esta característica del sistema');

        return $userDataTable->render('users.index');
    }

    /**
     * Show the form for creating a new UsuarioSucursal.
     *
     * @return Response
     */
    public function create()
    {

    }

    /**
     * Store a newly created UsuarioSucursal in storage.
     *
     * @param CreateUserRequest $request
     *
     * @return Response
     */
    public function store(CreateUserRequest $request)
    {

        if (!Auth::user()->role->permissions->create_user)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        $userData = $request->all();
        $password = $userData["password"];
        $userData["password"] = bcrypt($request->input('password'));

        try {

            $user = User::create($userData);

            Mail::to($user->email)->send(new UserCreated('Nuevo Usuario Panel Gruas San Juan', 'Nuevo Usuario de Administración ' . $user->name, $user->username, $password));

            return response()->json(["msg" => "Usuario registrado correctamente"], 200);


        } catch (QueryException $e) {

            return response()->json(["msg" => "Error al registrar el usuario " . $e->getMessage()], 400);

        } catch (\Swift_TransportException $e) {
            return response()->json(["msg" => "Usuario registrado correctamente, hubo un error al enviar el correo ", $e->getMessage()], 200);

        }

    }

    /**
     * Display the specified UsuarioSucursal.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {


    }

    /**
     * Show the form for editing the specified UsuarioSucursal.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        if (!Auth::user()->role->permissions->edit_user)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        $user = User::find($id);
        if (empty($user))
            return response()->json(["msg" => "Usuario no encontrado"], 404);

        return response()->json(["user" => $user], 200);


    }

    /**
     * Update the specified UsuarioSucursal in storage.
     *
     * @param  int $id
     * @param UpdateUserRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateUserRequest $request)
    {

        if (!Auth::user()->role->permissions->edit_user)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        $user = User::find($id);

        $userData = $request->all();


        unset($userData["email"]);
        unset($userData["password"]);

        if (empty($user))
            return response()->json(["msg" => "Usuario no encontrado"], 404);


        try {

            $user->update($userData);
            return response()->json(["msg" => "Usuario actualizado correctamente"], 200);


        } catch (QueryException $e) {

            return response()->json(["msg" => "Error al actualizar el usuario " . $e->getMessage()], 400);

        }

    }

    /**
     * Remove the specified UsuarioSucursal from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {

    }

    public function changePassword($id, ChangePasswordRequest $request)
    {
        if (!Auth::user()->role->permissions->edit_user)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        $user = User::find($id);
        if (empty($user))
            return response()->json(["msg" => "Usuario no encontrado"], 404);


        $user->email = $request->input('email');
        $user->password =Hash::make($request->input('password'));
        $user->save();



        try {
            Mail::to($user->email)->send(new UserCreated('Cambio de Credenciales Usuario Panel Gruas San Juan', 'Cambio de Contraseña ' . $user->name, $user->username, $request->input('password')));
            return response()->json(["msg" => "Credenciales del usuario actualizadas correctamente"], 200);

        } catch (\Swift_TransportException $e) {
            return response()->json(["msg" => "Credenciales del usuario actualizadas correctamente, hubo un error al enviar el correo ", $e->getMessage()], 200);

        }

    }

    public function setStatus($id, $status)
    {

        if (!Auth::user()->role->permissions->edit_user)
            return response()->json(["msg" => 'No tienes permisos para acceder a esta característica'], 401);

        $user = User::find($id);
        if (empty($user))
            return response()->json(["msg" => "Usuario no encontrado"], 404);

        $user->status = $status;
        $user->save();

        $textStatus = $status == 1 ? 'activado' : 'desactivado';
        return response()->json(["msg" => "Usuario " . $textStatus . " correctamente"], 200);

    }

}
