<?php

namespace App\Models;

use Eloquent as Model;

/**
 * Class Customer
 * @package App\Models
 * @version February 15, 2018, 10:42 am CST
 *
 * @property \Illuminate\Database\Eloquent\Collection ticketItem
 * @property \Illuminate\Database\Eloquent\Collection user
 * @property string rfc
 * @property string name
 * @property string email
 * @property string type
 */
class Customer extends Model
{

    public $table = 'customer';

    public $timestamps = false;



    public $fillable = [
        'rfc',
        'name',
        'email',
        'type',
        'status'
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'id' => 'integer',
        'rfc' => 'string',
        'name' => 'string',
        'email' => 'string',
        'type' => 'string',
        'status' => 'string'
    ];

    public static function rules($id = null)
    {

        $uniqueRfc = $id = null ? 'unique:customer,rfc' : 'unique:customer,rfc,' . $id . ',id';

        return [
            'rfc' => ['required', 'regex:/^([A-Za-z,Ññ,&]{3,4}([0-9]{2})(0[1-9]|1[0-2])(0[1-9]|1[0-9]|2[0-9]|3[0-1])[A-Za-z|\d]{3})$/', $uniqueRfc],
            'name' => 'required',
            'email' => 'required|email',
            'type' => 'required'
        ];
    }

    public static function messages()
    {
        return [
            'rfc.required' => "El rfc es obligatorio",
            'rfc.regex' => "El rfc no es válido",
            'rfc.unique' => "Este rfc ya ha sido registrado ",
            'name.required' => "La razón social es obligatoria",
            'email.required' => "El correo es obligatorio",
            'email.email' => "El correo no es válido",
            'type.required' => "Debe indicar si es persona física o moral"
        ];

    }


}
