<?php

namespace App\Models;

use Eloquent as Model;

/**
 * Class Filial
 * @package App\Models
 * @version January 29, 2018, 5:54 pm UTC
 *
 * @property \Illuminate\Database\Eloquent\Collection ticket
 * @property \Illuminate\Database\Eloquent\Collection User
 * @property string prefix
 * @property integer number_ticket
 * @property string name
 * @property string street
 * @property string ext
 * @property string int
 * @property string suburb
 * @property string council
 * @property string polity
 * @property string zip_code
 * @property string email
 * @property string phone
 * @property boolean status
 */
class Filial extends Model
{

    public $table = 'filial';

    public $timestamps = false;



    public $fillable = [
        'prefix',
        'number_ticket',
        'name',
        'street',
        'ext',
        'int',
        'suburb',
        'council',
        'polity',
        'zip_code',
        'email',
        'phone',
        'status'
    ];

    public function scopeActivated($query)
    {
        return $query->where('status', '=', 1);
    }

    public function scopeDeactivated($query)
    {
        return $query->where('status', '=', 0);
    }

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'id' => 'integer',
        'prefix' => 'string',
        'number_ticket' => 'integer',
        'name' => 'string',
        'street' => 'string',
        'ext' => 'string',
        'int' => 'string',
        'suburb' => 'string',
        'council' => 'string',
        'polity' => 'string',
        'zip_code' => 'string',
        'email' => 'string',
        'phone' => 'string',
        'status' => 'boolean'
    ];

    /**
     * Validation rules
     *
     * @var array
     */
    public static function rules($id = null)
    {

        $uniqueName = empty($id) ? 'unique:filial' : 'unique:filial,name,' . $id . ',id';
        $uniquePrefix = empty($id) ? 'unique:filial' : 'unique:filial,prefix,' . $id . ',id';

        return [
            'name' => 'required|' . $uniqueName,
            'prefix' => 'required|' . $uniquePrefix,
            'number_ticket' => 'required|integer',
            'street' => 'required',
            'ext' => 'required',
            'suburb' => 'required',
            'council' => 'required',
            'zip_code' => 'required',
            'polity' => 'required',
            'email' => 'required|email',
            'phone' => 'required',

        ];
    }

    public static function messages()
    {
        return [

            'name' => 'El nombre de la sucursal es obligatorio',
            'name.unique' => 'Esta sucursal ya ha sido registrada',
            'phone.required' => 'El teléfono de la sucursal es obligatorio',
            'email.required' => 'El correo de la sucursal es obligatorio',
            'email.email' => 'El correo de la sucursal no es válido',
            'number_ticket.required' => 'El nº de folio para ventas es obligatorio',
            'number_ticket.integer' => 'El nº de folio para ventas no es válido',
            'street.required' => 'La calle de la sucursal es obligatoria',
            'ext.required' => 'El nº exterior de la sucursal es obligatorio',
            'suburb.required' => 'La colonia de la sucursal es obligatoria',
            'zip_code.required' => 'El código postal de la sucursal es obligatorio',
            'council.required' => 'El municipio de la sucursal es obligatorio',
            'polity.required' => 'El estado de la sucursal es obligatorio',
        ];

    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function users()
    {
        return $this->hasMany(\App\Models\User::class);
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasOne
     **/
    public function filial_invoice()
    {
        return $this->hasOne(\App\Models\Filial::class,"id","related_filial");
    }
}
