<?php

namespace App\Models\Invoice;

use App\Models\Customer;
use Illuminate\Database\Eloquent\Model;
use Carbon\Carbon;

class GlobalInvoiceXml extends Model
{
    public static function createXml($filial, $request)
    {


        $company = ['rfc' => env('APP_ENV') == 'local' ? 'LAN7008173R5' : env('RFC'), 'name' => 'PERMISIONARIOS DE QUERÉTARO A.C.', 'regimen' => '621'];

        $cfdi = new \DOMDocument('1.0', 'UTF-8');
        $comprobante = $cfdi->createElement("cfdi:Comprobante");
        $emisor = $cfdi->createElement("cfdi:Emisor");
        $receptor = $cfdi->createElement("cfdi:Receptor");
        $concepto = $cfdi->createElement("cfdi:Conceptos");
        $nodeComplement = $cfdi->createElement('cfdi:Complemento');
        $stamp = $cfdi->createElement('tfd:TimbreFiscalDigital');

        //Comprobante
        $comprobante->setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
        $comprobante->setAttribute("xmlns:cfdi", "http://www.sat.gob.mx/cfd/3");
        $comprobante->setAttribute("xsi:schemaLocation", "http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv33.xsd");
        $comprobante->setAttribute("Version", "3.3");
        $comprobante->setAttribute("Fecha", Carbon::now()->format('Y-m-d') . 'T' . Carbon::now()->format('h:m:s'));
        $comprobante->setAttribute("Serie", $filial->prefix);
        $comprobante->setAttribute("Folio", $filial->number_ticket);
        $comprobante->setAttribute("LugarExpedicion", $filial->zip_code);
        $comprobante->setAttribute("Moneda", 'MXN');
        $comprobante->setAttribute("SubTotal", $request["ticket"]["subtotal"]);
        $comprobante->setAttribute("Total", $request["ticket"]["amount"]);
        $comprobante->setAttribute("TipoDeComprobante", 'I');
        $comprobante->setAttribute("TipoCambio", '1');
        $comprobante->setAttribute("FormaPago", empty($request["ticket"]["paymethod"])?'01':$request["ticket"]["paymethod"]);
        $comprobante->setAttribute("MetodoPago", 'PPD');
        $cfdi->appendChild($comprobante);

        //Emisor 
        $emisor->setAttribute("Rfc", env('APP_ENV') == 'local' ? 'LAN7008173R5' : $company["rfc"]);
        $emisor->setAttribute("Nombre", $company["name"]);
        $emisor->setAttribute("RegimenFiscal", $company["regimen"]);

        $comprobante->appendChild($emisor);

        if(isset($request["ticket"]["withFactura"])) {
            //Receptor
            $receptor_customer = Customer::find($request["customer_id"]);
            $receptor->setAttribute("Rfc", $receptor_customer["rfc"]);
            $receptor->setAttribute("Nombre", $receptor_customer["name"]);
            $receptor->setAttribute("UsoCFDI", 'P01');
            $comprobante->appendChild($receptor);

            $comprobante->appendChild($concepto);
            $addConcepto = $cfdi->createElement('cfdi:Concepto');

            $addConcepto->setAttribute("NoIdentificacion", "PQG");
            $addConcepto->setAttribute("Cantidad", 1);
            $addConcepto->setAttribute("Descripcion", "Servicios Varios");
            $addConcepto->setAttribute("ValorUnitario", $request["ticket"]["subtotal"]);
            $addConcepto->setAttribute("Importe", $request["ticket"]["subtotal"]);
            $addConcepto->setAttribute("ClaveProdServ", '01010101');
            $addConcepto->setAttribute("ClaveUnidad", 'E48');

            $concepto->appendChild($addConcepto);
            $impuestos = $cfdi->createElement("cfdi:Impuestos");
            $addConcepto->appendChild($impuestos);
            $traslados = $cfdi->createElement("cfdi:Traslados");
            $impuestos->appendChild($traslados);

            $traslado = $cfdi->createElement("cfdi:Traslado");
            $traslado->setAttribute("Impuesto", '002');
            $traslado->setAttribute("Base", $request["ticket"]["subtotal"]);
            $traslado->setAttribute("TasaOCuota", (floatval($request["ticket"]["iva"]) > 0) ? '0.160000' : '0.000000');
            $traslado->setAttribute("Importe", $request["ticket"]["iva"]);
            $traslado->setAttribute("TipoFactor", 'Tasa');
            $traslados->appendChild($traslado);



        }else {
            $receptor->setAttribute("Rfc", 'XAXX010101000');
            $receptor->setAttribute("Nombre", 'Ventas de mostrador');
            $receptor->setAttribute("UsoCFDI", 'P01');
            $comprobante->appendChild($receptor);

            //Conceptos
            $numTickets = count($request["ticket"]["items"]["id"]);

            for ($i = 0; $i < $numTickets; ++$i) {
                //if ($request["setTicket"][$i] == 'true') {
                $comprobante->appendChild($concepto);
                $addConcepto = $cfdi->createElement('cfdi:Concepto');

                $addConcepto->setAttribute("NoIdentificacion", $request["ticket"]["items"]["sku"][$i]);
                $addConcepto->setAttribute("Cantidad", 1);
                $addConcepto->setAttribute("Descripcion", "Venta");
                $addConcepto->setAttribute("ValorUnitario", $request["ticket"]["items"]["subtotal"][$i]);
                $addConcepto->setAttribute("Importe", $request["ticket"]["items"]["subtotal"][$i]);
                $addConcepto->setAttribute("ClaveProdServ", '01010101');
                $addConcepto->setAttribute("ClaveUnidad", 'E48');

                $concepto->appendChild($addConcepto);
                $impuestos = $cfdi->createElement("cfdi:Impuestos");
                $addConcepto->appendChild($impuestos);
                $traslados = $cfdi->createElement("cfdi:Traslados");
                $impuestos->appendChild($traslados);

                $traslado = $cfdi->createElement("cfdi:Traslado");
                $traslado->setAttribute("Impuesto", '002');
                $traslado->setAttribute("Base", $request["ticket"]["items"]["subtotal"][$i]);
                $traslado->setAttribute("TasaOCuota", (floatval($request["ticket"]["items"]["iva"][$i]) > 0) ? '0.160000' : '0.000000');
                $traslado->setAttribute("Importe", $request["ticket"]["items"]["iva"][$i]);
                $traslado->setAttribute("TipoFactor", 'Tasa');
                $traslados->appendChild($traslado);
                //}
            }

        }



        $iTotales = $cfdi->createElement("cfdi:Impuestos");
        $comprobante->appendChild($iTotales);
        $itraslados = $cfdi->createElement("cfdi:Traslados");
        $iTotales->appendChild($itraslados);

        $itraslado = $cfdi->createElement("cfdi:Traslado");
        $itraslado->setAttribute("Impuesto", '002');
        $itraslado->setAttribute("TasaOCuota", (floatval($request["ticket"]["iva"]) > 0)?'0.160000':'0.000000');
        $itraslado->setAttribute("Importe", $request["ticket"]["iva"]);
        $itraslado->setAttribute("TipoFactor", 'Tasa');
        $itraslados->appendChild($itraslado);

        $stamp->setAttribute('xmlns:tfd', 'http://www.sat.gob.mx/TimbreFiscalDigital');
        $stamp->setAttribute('xsi:schemaLocation', 'http://www.sat.gob.mx/TimbreFiscalDigital http://www.sat.gob.mx/sitio_internet/cfd/TimbreFiscalDigital/TimbreFiscalDigitalv11.xsd');
        $stamp->setAttribute('Version', '1.1');
        $stamp->setAttribute('UUID', '');
        $stamp->setAttribute('FechaTimbrado', '');
        $stamp->setAttribute('RfcProvCertif', '');
        $stamp->setAttribute('SelloCFD', '');
        $stamp->setAttribute('NoCertificadoSAT', '');
        $stamp->setAttribute('SelloSAT', '');

        $nodeComplement->appendChild($stamp);
        $comprobante->appendChild($nodeComplement);


        $file = storage_path() . '/app/filials/' . $filial->id . '/tickets/' . $filial->prefix . $filial->number_ticket . '.xml';
        $cfdi->save($file);

        return [
            'is_file' => is_file($file),
            "path" => $file
        ];

    }
}
