<?php

namespace App\Models\Invoice;

use App\Models\Filial;
use DOMDocument;
use Carbon\Carbon;
use Storage;
use Illuminate\Support\Facades\Auth;
use Ixudra\Curl\Facades\Curl;
use Log;



class InvoiceXml
{
    private $company;
    private $ticket;
    private $request;

    public function __construct($request, $ticket, $user)
    {
        $this->company = ['rfc' => env('APP_ENV') == 'local' ? 'AAA010101AAA' : env('RFC'), 'name' => 'PERMISIONARIOS DE QUERÉTARO A.C.', 'taxRegime' => '621'];
        $this->client = ['rfc' => 'XAXX010101000', 'name' => 'Ventas de Mostrador', 'cfdiUse' => 'G03'];
        $this->ticket = $ticket;

        if(( !isset($request["randomFolio"])  || !empty($this->ticket->invoice) || !empty($request["invoice"])) && $user->filial->related_filial !== null){
            $this->filial = $user->filial->filial_invoice;

        }else {
            $this->filial = $user->filial;
        }
        if(!empty($request)) {
            if (isset($request["remake"]) || isset($request["reinvoice"]))
                $this->request = $request;
            else
                $this->request = $request;
        }
        
        //foreach ($this->ticket->ticketItems as $item) {
        //}

    }


    public function generateXml()
    {
        
        $urlAPI = env('APP_ENV') == 'local' ? env("API_LOCAL") : env("API_PRODUCTION");
        $curl = Curl::to($urlAPI . '/items/satCodes/' . $this->ticket->ticketItems->item->sat_item_id . '/' . $this->ticket->ticketItems->item->sat_unit_id)
            ->asJson(true)
            ->withTimeout(1)
            ->withOption('RETURNTRANSFER','1')
            ->returnResponseObject()
            ->get();

        $this->ticket->ticketItems->sat_item_code = $curl->content["satItem"]["clave"];
        $this->ticket->ticketItems->sat_unit_code = $curl->content["satUnit"]["clave"];


        if( !isset($curl->content["satItem"]["clave"]) && !isset($curl->content["satUnit"]["clave"])){

            $this->ticket->ticketItems->sat_item_code = "78111804";
            $this->ticket->ticketItems->sat_unit_code = "E48";
        }


        $cfdi = new DOMDocument('1.0', 'UTF-8');
        $nodeTicket = $cfdi->createElement("cfdi:Comprobante");
        $nodeCompany = $cfdi->createElement("cfdi:Emisor");
        $nodeClient = $cfdi->createElement("cfdi:Receptor");
        $nodeItem = $cfdi->createElement("cfdi:Conceptos");
        $nodeComplement = $cfdi->createElement('cfdi:Complemento');
        $stamp = $cfdi->createElement('tfd:TimbreFiscalDigital');

        $commission = 0;
        $tipoCambio = 0;
        $coinKey = null;
        $desc_method_payment = "";
        $max_methodPayment = 0;
        $max_typeMethodPayment = "";
        if(!empty($this->ticket->ticketMethodPayment)) {
            foreach ($this->ticket->ticketMethodPayment as $method) {

                if($max_methodPayment < floatval($method->amount)){
                    $max_methodPayment = floatval($method->amount);
                    $max_typeMethodPayment = $method->methodPayment->type;
                }

                if ($method->methodPayment->commission > 0) {
                    $commission += round(floatval($method->amount * ($method->methodPayment->commission / 100)));

                    $desc_method_payment .= "($".$method->amount." ".$method->methodPayment->coin->key." ".strtoupper ($method->methodPayment->key). " )\n";

                    $desc_method_payment .= "+ %" . $method->methodPayment->commission . " de comisión de PagoMovil: $".$commission;
                }else{
                    $commission += 0;
                    $desc_method_payment .= "($".$method->amount." ".$method->methodPayment->coin->key." ".strtoupper ($method->methodPayment->key). " )\n";
                }

                $exchangeRate = floatval($method->exchangeRate->exchange_rate);
                $coinKey = $method->methodPayment->coin->key;
            }
        }

        $payMethod = "99";
        switch($max_typeMethodPayment){
            case "tarjeta": $payMethod = "28"; break;
            case "efectivo": $payMethod = "01"; break;
            case "credito": $payMethod = "04"; break;
            case "transferencia": $payMethod = "03";  break;
            case "aerolinea": $payMethod = "04"; break;
        }

        $nodeTicket->setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
        $nodeTicket->setAttribute("xmlns:cfdi", "http://www.sat.gob.mx/cfd/3");
        $nodeTicket->setAttribute("xsi:schemaLocation", "http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv33.xsd");
        $nodeTicket->setAttribute("Version", "3.3");
        $nodeTicket->setAttribute("Fecha", Carbon::now()->format('Y-m-d') . 'T' . Carbon::now()->format('h:m:s'));
        $nodeTicket->setAttribute("Serie", $this->filial->prefix);
        $nodeTicket->setAttribute("Folio", (( !isset($this->request["randomFolio"]))? ((!isset($this->request["invoice"]))? (!empty($this->ticket->invoice)? str_replace($this->filial->prefix,"",$this->ticket->invoice):str_replace($this->filial->prefix,"",$this->ticket->number_ticket)): (!empty($this->ticket->invoice)? str_replace($this->filial->prefix,"",$this->ticket->invoice): $this->filial->number_ticket)) : $this->request["randomFolio"]));
        $nodeTicket->setAttribute("LugarExpedicion", $this->filial->zip_code);
        $nodeTicket->setAttribute("Moneda", ($coinKey !== null)?$coinKey:'MXN');
        $nodeTicket->setAttribute("SubTotal", $this->ticket->subtotal + $commission); //$type == 'physical' ? $this->request["physical_ticket"]["subtotal"] : $this->request["moral_ticket"]["subtotal"]);
        $nodeTicket->setAttribute("Total", $this->ticket->amount + $commission); //$type == 'physical' ? $this->request["physical_ticket"]["total"] : $this->request["moral_ticket"]["total"]);
        $nodeTicket->setAttribute("TipoDeComprobante", 'I');
        $nodeTicket->setAttribute("TipoCambio", ($exchangeRate > 0)?$exchangeRate:'1');
        $nodeTicket->setAttribute("FormaPago", (!isset($this->request["ticket"]["paymethod"])? $payMethod:$this->request["ticket"]["paymethod"]));//Pendiente
        $nodeTicket->setAttribute("MetodoPago", 'PUE');//Pendiente

        $cfdi->appendChild($nodeTicket);

        $nodeCompany->setAttribute("Rfc", $this->company['rfc']);
        $nodeCompany->setAttribute("Nombre", $this->company['name']);
        $nodeCompany->setAttribute("RegimenFiscal", $this->company['taxRegime']);
        $nodeTicket->appendChild($nodeCompany);

        $nodeClient->setAttribute("Rfc", $this->client['rfc']);
        $nodeClient->setAttribute("Nombre", $this->client['name']);
        $nodeClient->setAttribute("UsoCFDI", $this->client['cfdiUse']);
        $nodeTicket->appendChild($nodeClient);

        $nodeTicket->appendChild($nodeItem);

        $descripcion = "Auto: " . $this->ticket->ticketItems->car->name . ", Destino: " . $this->ticket->ticketItems->item->destination .
            "\n Forma de Pago: " . $desc_method_payment. " ";

        if(isset($this->request["reinvoice"])){
            $descripcion .= (!empty($this->request["ticket_reinvoice"]))? $this->request["ticket_reinvoice"]: "";
        }

        //foreach ($this->ticket->ticketItems as $index => $ticketItem) {

            $subNodeItem = $cfdi->createElement('cfdi:Concepto');

            $subNodeItem->setAttribute("NoIdentificacion", (( !isset($this->request["randomFolio"]))? ((!isset($this->request["invoice"]))? (!empty($this->ticket->invoice)? $this->ticket->invoice:$this->ticket->number_ticket): (!empty($this->ticket->invoice)?$this->ticket->invoice: $this->filial->prefix.$this->filial->number_ticket) ) : $this->filial->prefix.$this->request["randomFolio"]));
            $subNodeItem->setAttribute("Cantidad", $this->ticket->ticketItems->quantity);
            $subNodeItem->setAttribute("Unidad", 'SERVICIO');
            $subNodeItem->setAttribute("Descripcion", $descripcion); //. ' ' . $this->request["ticket"]["job_order"]);
            $subNodeItem->setAttribute("ValorUnitario", $this->ticket->ticketItems->amount + $commission);//$type == 'physical' ? $this->request["physical_ticket"]["concept"]["subtotal"][$index] :
                //$this->request["moral_ticket"]["concept"]["subtotal"][$index]);
            $subNodeItem->setAttribute("Importe", $this->ticket->ticketItems->amount + $commission); //$type == 'physical' ? $this->request["physical_ticket"]["concept"]["subtotal"][$index] :
                //$this->request["moral_ticket"]["concept"]["subtotal"][$index]);
            $subNodeItem->setAttribute("ClaveProdServ", $this->ticket->ticketItems->sat_item_code);
            $subNodeItem->setAttribute("ClaveUnidad", $this->ticket->ticketItems->sat_unit_code);

            $nodeItem->appendChild($subNodeItem);

            //Total taxes
            $nodeTax = $cfdi->createElement("cfdi:Impuestos");
            $subNodeItem->appendChild($nodeTax);

            $nodeTransfers = $cfdi->createElement("cfdi:Traslados");
            $nodeTax->appendChild($nodeTransfers);

            //Fill iva Tax on physical or moral ticket
            $subNodeTransfer = $cfdi->createElement("cfdi:Traslado");
            $subNodeTransfer->setAttribute("Impuesto", '002');
            $subNodeTransfer->setAttribute("Base", $this->ticket->ticketItems->amount + $commission);//$type == 'physical' ? $this->request["physical_ticket"]["concept"]["subtotal"][$index] :
                //$this->request["moral_ticket"]["concept"]["subtotal"][$index]);
            $subNodeTransfer->setAttribute("TasaOCuota", (floatval($this->ticket->ticketItems->iva) > 0)?'0.160000':"0.000000");
            $subNodeTransfer->setAttribute("Importe", $this->ticket->ticketItems->iva);//$type == 'physical' ? $this->request["physical_ticket"]["concept"]["iva"][$index] :
                //$this->request["moral_ticket"]["concept"]["iva"][$index]);
            $subNodeTransfer->setAttribute("TipoFactor", "Tasa");
            $nodeTransfers->appendChild($subNodeTransfer);


            //Fill ret iva ta on moral ticket
            /*if ($type == 'moral') {
                /*$nodeRetentions = $cfdi->createElement("cfdi:Retenciones");
                $nodeTax->appendChild($nodeRetentions);

                $subNodeRetention = $cfdi->createElement("cfdi:Retencion");
                $subNodeRetention->setAttribute("Impuesto", '002');
                $subNodeRetention->setAttribute("Base", $this->request["moral_ticket"]["concept"]["subtotal"][$index]);
                $subNodeRetention->setAttribute("TasaOCuota", '0.040000');
                $subNodeRetention->setAttribute("Importe", $this->request["moral_ticket"]["concept"]["ret_iva"][$index]);
                $subNodeRetention->setAttribute("TipoFactor", 'Tasa');
                $nodeRetentions->appendChild($subNodeRetention);

            }*/
        //}



        //Fill total taxes' ticket
        $iTotales = $cfdi->createElement("cfdi:Impuestos");
        $nodeTicket->appendChild($iTotales);

        //Fill tax iva on physical/moral ticket
        $itraslados = $cfdi->createElement("cfdi:Traslados");
        $iTotales->appendChild($itraslados);

        $itraslado = $cfdi->createElement("cfdi:Traslado");
        $itraslado->setAttribute("Impuesto", '002');
        $itraslado->setAttribute("TasaOCuota", (floatval($this->ticket->ticketItems->iva) > 0)?'0.160000':"0.000000");
        $itraslado->setAttribute("Importe", $this->ticket->ticketItems->iva);//$type == 'physical' ? $this->request["physical_ticket"]["iva"] :
            //$this->request["moral_ticket"]["iva"]);
        $itraslado->setAttribute("TipoFactor", 'Tasa');
        $itraslados->appendChild($itraslado);

        $iTotales->setAttribute("TotalImpuestosTrasladados", $this->ticket->iva);//$type == 'physical' ? $this->request["physical_ticket"]["iva"] :
            //$this->request["moral_ticket"]["iva"]);



        //Fill ret iva tax on moral ticket
        /*if ($type == 'moral') {
            /*$iretenciones = $cfdi->createElement("cfdi:Retenciones");
            $iTotales->appendChild($iretenciones);

            $iretencion = $cfdi->createElement("cfdi:Retencion");
            $iretencion->setAttribute("Impuesto", '002');
            $iretencion->setAttribute("Importe", $this->request["moral_ticket"]["ret_iva"]);
            $iretenciones->appendChild($iretencion);
            $iTotales->setAttribute("TotalImpuestosRetenidos", $this->request["moral_ticket"]["ret_iva"]);
        }*/


        $stamp->setAttribute('xmlns:tfd', 'http://www.sat.gob.mx/TimbreFiscalDigital');
        $stamp->setAttribute('xsi:schemaLocation', 'http://www.sat.gob.mx/TimbreFiscalDigital http://www.sat.gob.mx/sitio_internet/cfd/TimbreFiscalDigital/TimbreFiscalDigitalv11.xsd');
        $stamp->setAttribute('Version', '1.1');
        $stamp->setAttribute('UUID', '');
        $stamp->setAttribute('FechaTimbrado', '');
        $stamp->setAttribute('RfcProvCertif', '');
        $stamp->setAttribute('SelloCFD', '');
        $stamp->setAttribute('NoCertificadoSAT', '');
        $stamp->setAttribute('SelloSAT', '');

        $nodeComplement->appendChild($stamp);
        $nodeTicket->appendChild($nodeComplement);

        $file_path = storage_path() . '/app/filials/' . $this->filial->id . '/tickets/';

        if(is_dir($file_path)) {
            $file = storage_path() . '/app/filials/' . $this->filial->id . '/tickets/' . ((!isset($this->request["randomFolio"])) ? ((!isset($this->request["invoice"]))? (!empty($this->ticket->invoice)? $this->ticket->invoice:$this->ticket->number_ticket): (!empty($this->ticket->invoice)?$this->ticket->invoice: $this->filial->prefix.$this->filial->number_ticket) ): $this->filial->prefix.$this->request["randomFolio"]) . '.xml';

            if(is_file($file)){
                $file = storage_path() . '/app/filials/' . $this->filial->id . '/tickets/' . (( !isset($this->request["randomFolio"])) ? ((!isset($this->request["invoice"]))? (!empty($this->ticket->invoice)? $this->ticket->invoice:$this->ticket->number_ticket): (!empty($this->ticket->invoice)?$this->ticket->invoice: $this->filial->prefix.$this->filial->number_ticket) ): $this->filial->prefix.$this->request["randomFolio"]) . '.xml';
                \File::Delete($file);
            }
            $cfdi->save($file);
            return is_file($file);
        }else{
            \File::makeDirectory($file_path, $mode = 0777, true, true);
        }

    }

}