<?php

namespace App\Models;

use Eloquent as Model;

/**
 * Class Item
 * @package App\Models
 * @version January 29, 2018, 5:56 pm UTC
 *
 * @property \Illuminate\Database\Eloquent\Collection ticket
 * @property \Illuminate\Database\Eloquent\Collection user
 * @property string name
 * @property integer sat_item_id
 * @property integer sat_unit_id
 * @property boolean status
 */
class Item extends Model
{

    public $table = 'item';

    public $timestamps = false;



    public $fillable = [
        'name',
        'sku',
        'measure_unit_id',
        'sat_item_id',
        'sat_unit_id',
        'price',
        'status'
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'id' => 'integer',
        'name' => 'string',
        'sku' => 'string',
        'measure_unit_id' => 'integer',
        'sat_item_id' => 'integer',
        'sat_unit_id' => 'integer',
        'price' => 'float',
        'status' => 'boolean'
    ];

    public function scopeActivated($query)
    {
        return $query->where('status', '=', 1);
    }

    public function scopeDeactivated($query)
    {
        return $query->where('status', '=', 0);
    }


    /**
     * Validation rules
     *
     * @var array
     */
    public static function rules($id = null)
    {
        $uniqueName = empty($id) ? 'unique:item' : 'unique:item,name,' . $id . ',id';
        $uniqueSku = empty($id) ? 'unique:item' : 'unique:item,sku,' . $id . ',id';

        return [
            'name' => 'required|' . $uniqueName,
            'sku' => 'required|' . $uniqueSku,
            'measure_unit_id' => 'required|integer',
            'sat_item_id' => 'required|integer',
            'sat_unit_id' => 'required|integer',
            'price' => 'required|numeric',
        ];

    }

    public static function messages()
    {

        return [
            'name.required' => 'El nombre del producto/servicio es obligatorio',
            'name.unique' => 'Este nombre de producto/servicio ya ha sido registrado',
            'sku.required' => 'El código del producto/servicio es obligatorio',
            'sku.unique' => 'Este código de producto/servicio ya ha sido registrado',
            'measure_unit_id.required' => 'La unidad del producto es obligatoria',
            'measure_unit_id.integer' => 'La unidad del producto proporcionada no es válida',
            'sat_item_id.required' => 'La clave del producto SAT es obligatoria',
            'sat_item_id.integer' => 'La clave del producto SAT proporcionada no es válida',
            'sat_unit_id.required' => 'La clave de unidad SAT proporcionada no es válida',
            'sat_unit_id.integer' => 'La clave de unidad SAT proporcionada no es válida',
            'price.required' => 'El precio del producto es obligatorio',
            'price.numeric' => 'El precio del producto no es válido',
        ];

    }

    public function measureUnit()
    {
        return $this->belongsTo(\App\Models\MeasureUnit::class);
    }



}
