<?php

namespace App\Models;

use Eloquent as Model;

/**
 * Class Role
 * @package App\Models
 * @version January 29, 2018, 5:56 pm UTC
 *
 * @property \Illuminate\Database\Eloquent\Collection ticket
 * @property \Illuminate\Database\Eloquent\Collection User
 * @property string name
 * @property string description
 * @property boolean status
 */
class Role extends Model
{

    public $table = 'role';

    public $timestamps = false;



    public $fillable = [
        'name',
        'description',
        'status'
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'id' => 'integer',
        'name' => 'string',
        'description' => 'string',
        'status' => 'boolean'
    ];

    public function scopeActivated($query)
    {
        return $query->where('status', '=', 1);
    }

    public function scopeDeactivated($query)
    {
        return $query->where('status', '=', 0);
    }


    /**
     * Validation rules
     *
     * @var array
     */
    public static function rules($id = null)
    {
        $unique = empty($id) ? 'unique:role' : 'unique:role,name,' . $id . ',id';

        return [
            'role.name' => 'required|' . $unique,
            'role_permission.general_report' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.cancel_invoice' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.cancel_ticket' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.sign_global' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.mark_bond' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.create_filial' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.edit_filial' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.create_item' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.edit_item' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.create_tax' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.edit_tax' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.create_user' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.edit_user' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.create_role' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.edit_role' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.create_unit' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.edit_unit' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.create_customer' => 'required|regex:/^[0-1]{1}$/',
            'role_permission.edit_customer' => 'required|regex:/^[0-1]{1}$/'

        ];

    }

    public static function messages()
    {
        return [
            'role.name.required' => 'El nombre del rol es obligatorio',
            'role.name.unique' => 'Este nombre de rol ya ha sido registrado',
            'role_permission.general_report.required' => 'El permiso de reporte general es obligatorio',
            'role_permission.general_report.regex' => 'El permiso de reporte general no es válido',
            'role_permission.cancel_invoice.required' => 'El permiso para cancelar facturas es obligatorio',
            'role_permission.cancel_invoice.regex' => 'El permiso para cancelar facturas no es válido',
            'role_permission.cancel_ticket.required' => 'El permiso para cancelar tickets es obligatorio',
            'role_permission.cancel_ticket.regex' => 'El permiso para cancelar tickets no es válido',
            'role_permission.sign_global.required' => 'El permiso para timbrar factura global es obligatorio',
            'role_permission.sign_global.regex' => 'El permiso para timbrar factura global no es válido',
            'role_permission.mark_bond.required' => 'El permiso para guardar depósitos de facturas es obligatorio',
            'role_permission.mark_bond.regex' => 'El permiso para guardar depósitos de facturas no es válido',
            'role_permission.create_filial.required' => 'El permiso para nueva sucursal es obligatorio',
            'role_permission.create_filial.regex' => 'El permiso para nueva sucursal no es válido',
            'role_permission.edit_filial.required' => 'El permiso para editar sucursal es obligatorio',
            'role_permission.edit_filial.regex' => 'El permiso para editar sucursal no es válido',
            'role_permission.create_item.required' => 'El permiso para nuevo producto es obligatorio',
            'role_permission.create_item.regex' => 'El permiso para nuevo producto no es válido',
            'role_permission.edit_item.required' => 'El permiso para editar producto es obligatorio',
            'role_permission.edit_item.regex' => 'El permiso para editar producto no es válido',
            'role_permission.create_tax.required' => 'El permiso para crear configuración de impuesto es obligatorio',
            'role_permission.create_tax.regex' => 'El permiso para crear configuración de impuesto no es válido',
            'role_permission.edit_tax.required' => 'El permiso para editar configuración de impuesto es obligatorio',
            'role_permission.edit_tax.regex' => 'El permiso para editar configuración de impuesto no es válido',
            'role_permission.create_role.required' => 'El permiso de nuevo rol es obligatorio',
            'role_permission.create_role.regex' => 'El permiso de nuevo rol no es válido',
            'role_permission.edit_role.required' => 'El permiso para editar rol es obligatorio',
            'role_permission.edit_role.regex' => 'El permiso para editar rol no es válido',
            'role_permission.create_user.required' => 'El permiso para crear usuario es obligatorio',
            'role_permission.create_user.regex' => 'El permiso para crear usuario no es válido',
            'role_permission.edit_user.required' => 'El permiso para editar usuarios es obligatorio',
            'role_permission.edit_user.regex' => 'El permiso para editar usuarios no es válido',
            'role_permission.create_unit.required' => 'El permiso para nueva unidad de medida es obligatorio',
            'role_permission.create_unit.regex' => 'El permiso para nueva unidad de medida no es válido',
            'role_permission.edit_unit.required' => 'El permiso para editar unidades de medida es obligatorio',
            'role_permission.edit_unit.regex' => 'El permiso para editar unidades de medida no es válido',
            'role_permission.create_customer.required' => 'El permiso para crear clientes es obligatorio',
            'role_permission.create_customer.regex' => 'El permiso para crear clientes no es válido',
            'role_permission.edit_customer.required' => 'El permiso para editar clientes es obligatorio',
            'role_permission.edit_customer.regex' => 'El permiso para editar clientes no es válido',



        ];

    }


    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function users()
    {
        return $this->hasMany(\App\Models\User::class);
    }

    public function permissions()
    {
        return $this->hasOne(\App\Models\RolePermission::class);
    }
}
