<?php

namespace App\Models;

use Eloquent as Model;

/**
 * Class TaxConfig
 * @package App\Models
 * @version January 29, 2018, 6:00 pm UTC
 *
 * @property \Illuminate\Database\Eloquent\Collection TaxItemConfig
 * @property \Illuminate\Database\Eloquent\Collection Ticket
 * @property \Illuminate\Database\Eloquent\Collection user
 * @property string name
 * @property boolean status
 */
class TaxConfig extends Model
{

    public $table = 'tax_config';

    public $timestamps = false;



    public $fillable = [
        'name',
        'x',
        'status'
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'id' => 'integer',
        'name' => 'string',
        'status' => 'boolean'
    ];

    public function scopeActivated($query)
    {
        return $query->where('status', '=', 1);
    }

    public function scopeDeactivated($query)
    {
        return $query->where('status', '=', 0);
    }

    /**
     * Validation rules
     *
     * @var array
     */
    public static function rules($id = null)
    {

        $unique = empty($id) ? 'unique:tax_config' : 'unique:tax_config,name,' . $id . ',id';

        return [
            'tax_config.name' => 'required|' . $unique,
        ];
    }

    public static function messages()
    {
        return [
            'tax_config.name.required' => 'El nombre de la configuración es obligatorio',
            'tax_config.name.unique' => 'Este nombre de configuración ya ha sido registrado',
        ];

    }




    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function taxItemConfigs()
    {
        return $this->hasMany(\App\Models\TaxItemConfig::class);
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function tickets()
    {
        return $this->hasMany(\App\Models\Ticket::class);
    }
}
