<?php

namespace App\Models;

use Eloquent as Model;
use Illuminate\Database\Eloquent\SoftDeletes;

/**
 * Class Ticket
 * @package App\Models
 * @version January 29, 2018, 6:03 pm UTC
 *
 * @property \App\Models\User user
 * @property \App\Models\TaxConfig taxConfig
 * @property \Illuminate\Database\Eloquent\Collection
 * @property integer user_id
 * @property integer id_tax_config
 * @property string|\Carbon\Carbon date
 * @property float amount
 */
class Ticket extends Model
{
    use SoftDeletes;

    public $table = 'ticket';

    const CREATED_AT = 'created_at';
    const UPDATED_AT = 'updated_at';


    protected $dates = ['deleted_at'];


    public $fillable = [
        'shift_user_id',
        'customer_id',
        'car_id',
        'job_order',
        'number_ticket',
        'invoice',
        'paymethod',
        'date',
        'date_bond',
        'subtotal',
        'subtotal_mx',
        'iva',
        'iva_mx',
        'ret_iva',
        'amount',
        'amount_mx',
        'status',
        'note',
        'pay'
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'id' => 'integer',
        'shift_user_id' => 'integer',
        'customer_id' => 'integer',
        'car_id' => 'integer',
        'job_order' => 'string',
        'number_ticket' => 'string',
        'invoice' => 'string',
        'paymethod' => 'string',
        'date' => 'datetime',
        'date_bond' => 'date_format:Y-m-d',
        'number' => 'integer',
        'subtotal' => 'float',
        'subtotal_mx' => 'float',
        'iva' => 'float',
        'iva_mx' => 'float',
        'ret_iva' => 'float',
        'amount' => 'float',
        'amount_mx' => 'float',
        'status' => 'string',
        'note' => 'string',
        'pay' => 'integer'
    ];

    /**
     * Validation rules
     *
     * @var array
     */
    public static $rules = [];

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function userShift()
    {
        return $this->belongsTo(\App\Models\UserShift::class, 'shift_user_id', 'id');
    }


    public function customer()
    {
        return $this->belongsTo(\App\Models\Customer::class);
    }

    public function ticketItems()
    {
        return $this->hasOne(\App\Models\TicketItem::class);
    }

    public function ticketMethodPayment()
    {
        return $this->hasMany(\App\Models\TicketMethodPayment::class);
    }

    public function ticketGlobalItems()
    {
        return $this->hasMany(\App\Models\TicketGlobalItem::class, 'ticket_global_id', 'id');
    }

    public function ticketPay()
    {
        return $this->hasOne(\App\Models\TicketPay::class, 'ticket_id', 'id');
    }


}
