<?php

namespace App\Models\Ticket;

use DOMDocument;
use Carbon\Carbon;
use Mike42\Escpos\EscposImage;
use Mike42\Escpos\PrintConnectors\FilePrintConnector;
use Mike42\Escpos\PrintConnectors\WindowsPrintConnector;
use Mike42\Escpos\Printer;
use Storage;
use Exception;
use Illuminate\Support\Facades\Auth;
use Ixudra\Curl\Facades\Curl;



class TicketPrint
{
    private $ticket;
    private $request;

    public static function printTicket($ticket){



        //$connector = new WindowsPrintConnector("LPT1");
        //$connector = new WindowsPrintConnector("EPSON TM-T20II Receipt");
        //$connector = new FilePrintConnector("/dev/usb/lp1");
        //$connector = new FilePrintConnector("php://stdout");
        $connector = new FilePrintConnector("ESDPRT001");
        $printer = new Printer($connector);

        $printer->setJustification(Printer::JUSTIFY_CENTER);
        $printer->setEmphasis(true);
        $printer->text($ticket->userShift->user->filial->name. "\n");
        $printer->setEmphasis(false);

        $printer->text("RFC: PQU031204TQ3 \n");
        $printer->text($ticket->userShift->user->filial->street." ".$ticket->userShift->user->filial->ext." ".
                $ticket->userShift->user->filial->int." ".$ticket->userShift->user->filial->suburb." \n");

        $printer->text($ticket->userShift->user->filial->council." ,".$ticket->userShift->user->filial->polity."C.P. ".
            $ticket->userShift->user->filial->zip_code. "\n");


        $printer->text("Telefono: ".$ticket->userShift->user->filial->phone. "\n");
        $printer->text("Correo: ".$ticket->userShift->user->filial->email. "\n");

        $printer->text( "\n");

        $printer->setJustification(Printer::JUSTIFY_LEFT);
        $line = sprintf('%19.19s %19.19s', "Día: ".$ticket->date->format('Y-m-d'), "Hora: ".$ticket->date->format('h:m:s a'));
        $printer->text($line);
        $printer->text( "\n");
        $printer ->pulse();
        $printer->setJustification(Printer::JUSTIFY_LEFT);
        $printer->text("Cajero: ");
        $printer->setEmphasis(true);
        $printer->text($ticket->userShift->user->name. "\n");
        $printer->setEmphasis(false);

        $printer->text("Ticket No: ".$ticket->number_ticket. "\n");
        $printer->text("\n");

        $line = sprintf('%5.5s %20.20s %10.10s %10.10s', "CANT.", "DESCRIPCION", "PRECIO", "TOTAL");
        $printer->text($line);
        $printer->text("\n");

        $total_concepto = number_format($ticket->ticketItems->amount,2);
        $precio_concepto = number_format($ticket->ticketItems->price,2);

        $explode_descripcion = explode(",",$ticket->ticketItems->description, 20);
        $divide_description = str_split($explode_descripcion[1], 20);

        $line = sprintf('%5u %20.20s %10.10s %10.10s', $ticket->ticketItems->quantity, $explode_descripcion[0],
            $precio_concepto, $total_concepto);
        $printer->text($line);
        $printer->text("\n");


        foreach ($divide_description as $t => $text) {

            $line = sprintf('%5.5s %20.20s %10.10s %10.10s', "", $divide_description[$t],
                "", "");
            $printer->text($line);
            $printer->text("\n");

        }

        $total = $total_concepto;

        if(!empty($ticket->ticketMethodPayment)){
            foreach ($ticket->ticketMethodPayment as $method){
                if($method->methodPayment->commission > 0){
                    $commission =number_format( $method->amount * ($method->methodPayment->commission/100),2);
                    $total += $commission;

                    $line = sprintf('%5.5s %20.20s %8.2f %8.2f', "1", "Comisión PagoMovil %". $method->methodPayment->commission,
                        $commission, $commission);
                    $printer->text($line);
                    $printer->text("\n");
                }
            }
        }


        $line = sprintf('%4.4s %11.11s %15.15s %15.15s', "TOTAL", "", "", $total);
        $printer->text($line);

        $printer->text("\n");

        $printer->setJustification(Printer::JUSTIFY_CENTER);
        $printer->text("Gracias por su compra");
        $printer->text("\n");
        $printer->text("Para obtener su factura electrónica ingrese a la siguiente dirección:");
        $printer->text("\n");
        $printer->text(route('tickets.invoice'));
        $printer->text("\n");

        $printer -> cut();
        $printer -> close();


        return true;

    }

}