<?php

namespace App\Models;

use Eloquent as Model;

/**
 * Class User
 * @package App\Models
 * @version January 29, 2018, 6:05 pm UTC
 *
 * @property \App\Models\Filial filial
 * @property \App\Models\Role role
 * @property \Illuminate\Database\Eloquent\Collection Ticket
 * @property integer filial_id
 * @property integer role_id
 * @property integer name
 * @property string username
 * @property string password
 * @property string email
 * @property string phone
 * @property boolean status
 */
class User extends Model
{

    public $table = 'user';
    
    public $timestamps = false;



    public $fillable = [
        'filial_id',
        'role_id',
        'name',
        'username',
        'password',
        'email',
        'phone',
        'status',
        'related_filial'
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'id' => 'integer',
        'filial_id' => 'integer',
        'role_id' => 'integer',
        'name' => 'string',
        'username' => 'string',
        'password' => 'string',
        'email' => 'string',
        'phone' => 'string',
        'status' => 'integer',
        'related_filial' => 'integer'
    ];

    public function scopeActivated($query)
    {
        return $query->where('status', '=', 1);
    }

    public function scopeDeactivated($query)
    {
        return $query->where('status', '=', 0);
    }


    /**
     * Validation rules
     *
     * @var array
     */
    public static function rules($id = null)
    {
        $uniqueEmail = empty($id) ? 'unique:user' : 'unique:user,email,' . $id . ',id';
        $uniqueUsername = empty($id) ? 'unique:user' : 'unique:user,username,' . $id . ',id';
        $password = empty($id) ? '|required' : '';
        $email = empty($id) ? '|required|email|' . $uniqueEmail : '';
        $username = empty($id) ? '|required|' . $uniqueUsername: '';
        

        return [
            'filial_id' => 'required|integer',
            'role_id' => 'required|integer',
            'name' => 'required',
            'username' => $username,
            'email' => $email,
            'password' => 'min:6' . $password,
        ];


    }

    public static function messages()
    {
        return [
            'filial_id.required' => 'La sucursal a la que pertenece el usuario es obligatoria',
            'filial_id.integer' => 'La sucursal a la que pertenece el usuario no es válida',
            'role_id.required' => 'El rol del usuario es obligatorio',
            'role_id.integer' => 'El rol del usuario no es válido',
            'name.required' => 'El nombre de la persona  es obligatorio',
            'username.required' => 'El nombre del usuario  es obligatorio',
            'username.unique' => 'Este nombre de usuario ya ha sido registrado',
            'email.required' => 'El correo del usuario  es obligatorio',
            'email.email' => 'El correo del usuario no es válido',
            'email.unique' => 'Este correo ya ha sido registrado',
            'password.required' => 'El password es obligatorio',
            'password.min' => 'El password debe tener al menos 6 caracteres',
        ];

    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function filial()
    {
        return $this->belongsTo(\App\Models\Filial::class);
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function role()
    {
        return $this->belongsTo(\App\Models\Role::class);
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function tickets()
    {
        return $this->hasMany(\App\Models\Ticket::class);
    }
}
